from faker.typing import Country

from .. import Provider as DateTimeProvider


class Provider(DateTimeProvider):
    DAY_NAMES = {
        "0": "Воскресенье",
        "1": "Понедельник",
        "2": "Вторник",
        "3": "Среда",
        "4": "Четверг",
        "5": "Пятница",
        "6": "Суббота",
    }

    MONTH_NAMES = {
        "01": "Январь",
        "02": "Февраль",
        "03": "Март",
        "04": "Апрель",
        "05": "Май",
        "06": "Июнь",
        "07": "Июль",
        "08": "Август",
        "09": "Сентябрь",
        "10": "Октябрь",
        "11": "Ноябрь",
        "12": "Декабрь",
    }

    # Timezone names are based on Wiki list, source: https://ru.wikipedia.org/wiki/Список_часовых_поясов_по_странам
    countries = [
        Country(
            timezones=["Андорра (UTC+01)"],
            alpha_2_code="AD",
            alpha_3_code="AND",
            continent="Европа",
            name="Андорра",
            capital="Андорра-ла-Велья",
        ),
        Country(
            timezones=["Афганистан (UTC+04:30)"],
            alpha_2_code="AF",
            alpha_3_code="AFG",
            continent="Азия",
            name="Афганистан",
            capital="Кабул",
        ),
        Country(
            timezones=["Антигуа и Барбуда (UTC-04)"],
            alpha_2_code="AG",
            alpha_3_code="ATG",
            continent="Северная Америка",
            name="Антигуа и Барбуда",
            capital="Сент-Джонс",
        ),
        Country(
            timezones=["Албания (UTC+01)"],
            alpha_2_code="AL",
            alpha_3_code="ALB",
            continent="Европа",
            name="Албания",
            capital="Тирана",
        ),
        Country(
            timezones=["Армения (UTC+04)"],
            alpha_2_code="AM",
            alpha_3_code="ARM",
            continent="Азия",
            name="Армения",
            capital="Ереван",
        ),
        Country(
            timezones=["Ангола (UTC+01)"],
            alpha_2_code="AO",
            alpha_3_code="AGO",
            continent="Африка",
            name="Ангола",
            capital="Луанда",
        ),
        Country(
            timezones=["Аргентина (UTC-03)"],
            alpha_2_code="AR",
            alpha_3_code="ARG",
            continent="Южная Америка",
            name="Аргентина",
            capital="Буэнос Айрес",
        ),
        Country(
            timezones=["Австрия (UTC+01)"],
            alpha_2_code="AT",
            alpha_3_code="AUT",
            continent="Европа",
            name="Австрия",
            capital="Вена",
        ),
        Country(
            timezones=[
                "Австралия (UTC+05)",
                "Австралия (UTC+06:30)",
                "Австралия (UTC+07)",
                "Австралия (UTC+08)",
                "Австралия (UTC+9:30)",
                "Австралия (UTC+10)",
                "Австралия (UTC+10:30)",
                "Австралия (UTC+11:30)",
            ],
            alpha_2_code="AU",
            alpha_3_code="AUS",
            continent="Океания",
            name="Австралия",
            capital="Канберра",
        ),
        Country(
            timezones=["Азербайджан (UTC+04)"],
            alpha_2_code="AZ",
            alpha_3_code="AZE",
            continent="Азия",
            name="Азербайджан",
            capital="Баку",
        ),
        Country(
            timezones=["Барбадос (UTC-04)"],
            alpha_2_code="BB",
            alpha_3_code="BRB",
            continent="Северная Америка",
            name="Барбадос",
            capital="Бриджтаун",
        ),
        Country(
            timezones=["Бангладеш (UTC+06)"],
            alpha_2_code="BD",
            alpha_3_code="BGD",
            continent="Азия",
            name="Бангладеш",
            capital="Дака",
        ),
        Country(
            timezones=["Бельгия (UTC+01)"],
            alpha_2_code="BE",
            alpha_3_code="BEL",
            continent="Европа",
            name="Бельгия",
            capital="Брюссель",
        ),
        Country(
            timezones=["Буркина-Фасо (UTC)"],
            alpha_2_code="BF",
            alpha_3_code="BFA",
            continent="Африка",
            name="Буркина-Фасо",
            capital="Уагадугу",
        ),
        Country(
            timezones=["Болгария (UTC+02)"],
            alpha_2_code="BG",
            alpha_3_code="BGR",
            continent="Европа",
            name="Болгария",
            capital="София",
        ),
        Country(
            timezones=["Бахрейн (UTC+03)"],
            alpha_2_code="BH",
            alpha_3_code="BHR",
            continent="Азия",
            name="Бахрейн",
            capital="Манама",
        ),
        Country(
            timezones=["Бурунди (UTC+02)"],
            alpha_2_code="BI",
            alpha_3_code="BDI",
            continent="Африка",
            name="Бурунди",
            capital="Гитега",
        ),
        Country(
            timezones=["Бенин (UTC+01)"],
            alpha_2_code="BJ",
            alpha_3_code="BEN",
            continent="Африка",
            name="Бенин",
            capital="Порто-Ново",
        ),
        Country(
            timezones=["Бруней (UTC+08)"],
            alpha_2_code="BN",
            alpha_3_code="BRN",
            continent="Азия",
            name="Бруней",
            capital="Бандар-Сери-Бегаван",
        ),
        Country(
            timezones=["Боливия (UTC-04)"],
            alpha_2_code="BO",
            alpha_3_code="BOL",
            continent="Южная Америка",
            name="Боливия",
            capital="Сукре",
        ),
        Country(
            timezones=[
                "Бразилия (UTC-05)",
                "Бразилия (UTC-04)",
                "Бразилия (UTC-03)",
                "Бразилия (UTC-02)",
            ],
            alpha_2_code="BR",
            alpha_3_code="BRA",
            continent="Южная Америка",
            name="Бразилия",
            capital="Бразилиа",
        ),
        Country(
            timezones=["Багамские Острова (UTC-05)"],
            alpha_2_code="BS",
            alpha_3_code="BHS",
            continent="Северная Америка",
            name="Багамские Острова",
            capital="Нассау",
        ),
        Country(
            timezones=["Бутан (UTC+06)"],
            alpha_2_code="BT",
            alpha_3_code="BTN",
            continent="Азия",
            name="Бутан",
            capital="Тхимпху",
        ),
        Country(
            timezones=["Ботсвана (UTC+02)"],
            alpha_2_code="BW",
            alpha_3_code="BWA",
            continent="Африка",
            name="Ботсвана",
            capital="Габороне",
        ),
        Country(
            timezones=["Белоруссия (UTC+03)"],
            alpha_2_code="BY",
            alpha_3_code="BLR",
            continent="Европа",
            name="Белоруссия",
            capital="Минск",
        ),
        Country(
            timezones=["Белиз (UTC-06)"],
            alpha_2_code="BZ",
            alpha_3_code="BLZ",
            continent="Северная Америка",
            name="Белиз",
            capital="Бельмопан",
        ),
        Country(
            timezones=[
                "Канада (UTC-08)",
                "Канада (UTC-07)",
                "Канада (UTC-06)",
                "Канада (UTC-05)",
                "Канада (UTC-04)",
                "Канада (UTC-03:30)",
            ],
            alpha_2_code="CA",
            alpha_3_code="CAN",
            continent="Северная Америка",
            name="Канада",
            capital="Оттава",
        ),
        Country(
            timezones=[
                "Демократическая Республика Конго (UTC+01)",
                "Демократическая Республика Конго (UTC+02)",
            ],
            alpha_2_code="CD",
            alpha_3_code="COD",
            continent="Африка",
            name="Демократическая Республика Конго",
            capital="Киншаса",
        ),
        Country(
            timezones=["Республика Конго (UTC+01)"],
            alpha_2_code="CG",
            alpha_3_code="COG",
            continent="Африка",
            name="Руспублика Конго",
            capital="Браззавиль",
        ),
        Country(
            timezones=["Кот-д'Ивуар (UTC)"],
            alpha_2_code="CI",
            alpha_3_code="CIV",
            continent="Африка",
            name="Кот-д'Ивуар",
            capital="Ямусукро",
        ),
        Country(
            timezones=["Чили (UTC-06)", "Чили (UTC-04)"],
            alpha_2_code="CL",
            alpha_3_code="CHL",
            continent="Южная Америка",
            name="Чили",
            capital="Сантьяго",
        ),
        Country(
            timezones=["Камерун (UTC+01)"],
            alpha_2_code="CM",
            alpha_3_code="CMR",
            continent="Африка",
            name="Камерун",
            capital="Яунде",
        ),
        Country(
            timezones=["Китай (UTC+08)"],
            alpha_2_code="CN",
            alpha_3_code="CHN",
            continent="Азия",
            name="Китайская Народная Республика",
            capital="Пекин",
        ),
        Country(
            timezones=["Колумбия (UTC-05)"],
            alpha_2_code="CO",
            alpha_3_code="COL",
            continent="Южная Америка",
            name="Колумбия",
            capital="Богота",
        ),
        Country(
            timezones=["Коста-Рика (UTC-06)"],
            alpha_2_code="CR",
            alpha_3_code="CRI",
            continent="Северная Америка",
            name="Коста-Рика",
            capital="Сан-Хосе",
        ),
        Country(
            timezones=["Куба (UTC-05)"],
            alpha_2_code="CU",
            alpha_3_code="CUB",
            continent="Северная Америка",
            name="Куба",
            capital="Гавана",
        ),
        Country(
            timezones=["Кабо-Верде (UTC-01)"],
            alpha_2_code="CV",
            alpha_3_code="CPV",
            continent="Африка",
            name="Кабо-Верде",
            capital="Прая",
        ),
        Country(
            timezones=["Кипр (UTC+02)"],
            alpha_2_code="CY",
            alpha_3_code="CYP",
            continent="Азия",
            name="Кипр",
            capital="Никосия",
        ),
        Country(
            timezones=["Чехия (UTC+01)"],
            alpha_2_code="CZ",
            alpha_3_code="CZE",
            continent="Европа",
            name="Чехия",
            capital="Прага",
        ),
        Country(
            timezones=["Германия (UTC+01)"],
            alpha_2_code="DE",
            alpha_3_code="DEU",
            continent="Европа",
            name="Германия",
            capital="Берлин",
        ),
        Country(
            timezones=["Джибути (UTC+03)"],
            alpha_2_code="DJ",
            alpha_3_code="DJI",
            continent="Африка",
            name="Джибути",
            capital="Джибути",
        ),
        Country(
            timezones=["Дания (UTC+01)"],
            alpha_2_code="DK",
            alpha_3_code="DNK",
            continent="Европа",
            name="Дания",
            capital="Копенгаген",
        ),
        Country(
            timezones=["Доминика (UTC-04)"],
            alpha_2_code="DM",
            alpha_3_code="DMA",
            continent="Северная Америка",
            name="Доминика",
            capital="Розо",
        ),
        Country(
            timezones=["Доминиканская Республика (UTC-04)"],
            alpha_2_code="DO",
            alpha_3_code="DOM",
            continent="Северная Америка",
            name="Доминиканская Республика",
            capital="Санто-Доминго",
        ),
        Country(
            timezones=["Эквадор (UTC-06)", "Эквадор (UTC-05)"],
            alpha_2_code="EC",
            alpha_3_code="ECU",
            continent="Южная Америка",
            name="Эквадор",
            capital="Кито",
        ),
        Country(
            timezones=["Эстония (UTC+02)"],
            alpha_2_code="EE",
            alpha_3_code="EST",
            continent="Европа",
            name="Эстония",
            capital="Таллинн",
        ),
        Country(
            timezones=["Египет (UTC+02)"],
            alpha_2_code="EG",
            alpha_3_code="EGY",
            continent="Африка",
            name="Египет",
            capital="Каир",
        ),
        Country(
            timezones=["Эритрея (UTC+03)"],
            alpha_2_code="ER",
            alpha_3_code="ERI",
            continent="Африка",
            name="Эритрея",
            capital="Асмэра",
        ),
        Country(
            timezones=["Эфиопия (UTC+03)"],
            alpha_2_code="ET",
            alpha_3_code="ETH",
            continent="Африка",
            name="Эфиопия",
            capital="Аддис-Абеба",
        ),
        Country(
            timezones=["Финляндия (UTC+02)"],
            alpha_2_code="FI",
            alpha_3_code="FIN",
            continent="Европа",
            name="Финляндия",
            capital="Хельсинки",
        ),
        Country(
            timezones=["Фиджи (UTC+12)"],
            alpha_2_code="FJ",
            alpha_3_code="FJI",
            continent="Океания",
            name="Фиджи",
            capital="Сува",
        ),
        Country(
            timezones=["Франция (UTC+01)"],
            alpha_2_code="FR",
            alpha_3_code="FRA",
            continent="Европа",
            name="Франция",
            capital="Париж",
        ),
        Country(
            timezones=["Габон (UTC+01)"],
            alpha_2_code="GA",
            alpha_3_code="GAB",
            continent="Африка",
            name="Габон",
            capital="Либревиль",
        ),
        Country(
            timezones=["Грузия (UTC+04)"],
            alpha_2_code="GE",
            alpha_3_code="GEO",
            continent="Азия",
            name="Грузия",
            capital="Тбилиси",
        ),
        Country(
            timezones=["Гана (UTC)"],
            alpha_2_code="GH",
            alpha_3_code="GHA",
            continent="Африка",
            name="Гана",
            capital="Аккра",
        ),
        Country(
            timezones=["Гамбия (UTC)"],
            alpha_2_code="GM",
            alpha_3_code="GMB",
            continent="Африка",
            name="Гамбия",
            capital="Банджул",
        ),
        Country(
            timezones=["Гвинея (UTC)"],
            alpha_2_code="GN",
            alpha_3_code="GIN",
            continent="Африка",
            name="Гвинея",
            capital="Конакри",
        ),
        Country(
            timezones=["Греция (UTC+02)"],
            alpha_2_code="GR",
            alpha_3_code="GRC",
            continent="Европа",
            name="Греция",
            capital="Афины",
        ),
        Country(
            timezones=["Гватемала (UTC-06)"],
            alpha_2_code="GT",
            alpha_3_code="GTM",
            continent="Северная Америка",
            name="Гватемала",
            capital="Гватемала",
        ),
        Country(
            timezones=["Гаити (UTC-05)"],
            alpha_2_code="HT",
            alpha_3_code="HTI",
            continent="Северная Америка",
            name="Гаити",
            capital="Порт-о-Пренс",
        ),
        Country(
            timezones=["Гвинея-Бисау (UTC)"],
            alpha_2_code="GW",
            alpha_3_code="GNB",
            continent="Африка",
            name="Гвинея-Бисау",
            capital="Бисау",
        ),
        Country(
            timezones=["Гайана (UTC-04)"],
            alpha_2_code="GY",
            alpha_3_code="GUY",
            continent="Южная Америка",
            name="Гайана",
            capital="Джорджтаун",
        ),
        Country(
            timezones=["Гондурас (UTC-06)"],
            alpha_2_code="HN",
            alpha_3_code="HND",
            continent="Северная Америка",
            name="Гондурас",
            capital="Тегусигальпа",
        ),
        Country(
            timezones=["Венгрия (UTC+01)"],
            alpha_2_code="HU",
            alpha_3_code="HUN",
            continent="Европа",
            name="Венгрия",
            capital="Будапешт",
        ),
        Country(
            timezones=[
                "Индонезия (UTC+07)",
                "Индонезия (UTC+08)",
                "Индонезия (UTC+09)",
            ],
            alpha_2_code="ID",
            alpha_3_code="IDN",
            continent="Азия",
            name="Индонезия",
            capital="Джакарта",
        ),
        Country(
            timezones=["Ирландия (UTC)"],
            alpha_2_code="IE",
            alpha_3_code="IRL",
            continent="Европа",
            name="Ирландия",
            capital="Дублин",
        ),
        Country(
            timezones=["Израиль (UTC+02)"],
            alpha_2_code="IL",
            alpha_3_code="ISR",
            continent="Азия",
            name="Израиль",
            capital="Иерусалим",
        ),
        Country(
            timezones=["Индия (UTC+05:30"],
            alpha_2_code="IN",
            alpha_3_code="IND",
            continent="Азия",
            name="Индия",
            capital="Дели",
        ),
        Country(
            timezones=["Ирак (UTC+03)"],
            alpha_2_code="IQ",
            alpha_3_code="IRQ",
            continent="Азия",
            name="Ирак",
            capital="Багдад",
        ),
        Country(
            timezones=["Иран (UTC+03:30)"],
            alpha_2_code="IR",
            alpha_3_code="IRN",
            continent="Азия",
            name="Иран",
            capital="Тегеран",
        ),
        Country(
            timezones=["Исландия (UTC)"],
            alpha_2_code="IS",
            alpha_3_code="ISL",
            continent="Европа",
            name="Исландия",
            capital="Рейкьявик",
        ),
        Country(
            timezones=["Италия (UTC+01)"],
            alpha_2_code="IT",
            alpha_3_code="ITA",
            continent="Европа",
            name="Италия",
            capital="Рим",
        ),
        Country(
            timezones=["Ямайка (UTC-05)"],
            alpha_2_code="JM",
            alpha_3_code="JAM",
            continent="Северная Америка",
            name="Ямайка",
            capital="Кингстон",
        ),
        Country(
            timezones=["Иордания (UTC+02)"],
            alpha_2_code="JO",
            alpha_3_code="JOR",
            continent="Азия",
            name="Иордания",
            capital="Амман",
        ),
        Country(
            timezones=["Япония (UTC+09)"],
            alpha_2_code="JP",
            alpha_3_code="JPN",
            continent="Азия",
            name="Япония",
            capital="Токио",
        ),
        Country(
            timezones=["Кения (UTC+03)"],
            alpha_2_code="KE",
            alpha_3_code="KEN",
            continent="Африка",
            name="Кения",
            capital="Найроби",
        ),
        Country(
            timezones=["Киргизия (UTC+06)"],
            alpha_2_code="KG",
            alpha_3_code="KGZ",
            continent="Азия",
            name="Киргизия",
            capital="Бишкек",
        ),
        Country(
            timezones=[
                "Кирибати (UTC+12)",
                "Кирибати (UTC+13)",
                "Кирибати (UTC+14)",
            ],
            alpha_2_code="KI",
            alpha_3_code="KIR",
            continent="Океания",
            name="Кирибати",
            capital="Южная Тарава",
        ),
        Country(
            timezones=["КНДР (UTC+09)"],
            alpha_2_code="KP",
            alpha_3_code="PRK",
            continent="Азия",
            name="КНДР",
            capital="Пхеньян",
        ),
        Country(
            timezones=["Республика Корея (UTC+09)"],
            alpha_2_code="KR",
            alpha_3_code="KOR",
            continent="Азия",
            name="Республика Корея",
            capital="Сеул",
        ),
        Country(
            timezones=["Кувейт (UTC+03)"],
            alpha_2_code="KW",
            alpha_3_code="KWT",
            continent="Азия",
            name="Кувейт",
            capital="Эль-Кувейт",
        ),
        Country(
            timezones=["Ливан (UTC+02)"],
            alpha_2_code="LB",
            alpha_3_code="LBN",
            continent="Азия",
            name="Ливан",
            capital="Бейрут",
        ),
        Country(
            timezones=["Лихтенштейн (UTC+01)"],
            alpha_2_code="LI",
            alpha_3_code="LIE",
            continent="Европа",
            name="Лихтенштейн",
            capital="Вадуц",
        ),
        Country(
            timezones=["Либерия (UTC)"],
            alpha_2_code="LR",
            alpha_3_code="LBR",
            continent="Африка",
            name="Либерия",
            capital="Монровия",
        ),
        Country(
            timezones=["Лесото (UTC+02)"],
            alpha_2_code="LS",
            alpha_3_code="LSO",
            continent="Африка",
            name="Лесото",
            capital="Масеру",
        ),
        Country(
            timezones=["Литва (UTC+02)"],
            alpha_2_code="LT",
            alpha_3_code="LTU",
            continent="Европа",
            name="Литва",
            capital="Вильнюс",
        ),
        Country(
            timezones=["Люксембург (UTC+01)"],
            alpha_2_code="LU",
            alpha_3_code="LUX",
            continent="Европа",
            name="Люксембург",
            capital="Люксембург",
        ),
        Country(
            timezones=["Латвия (UTC+02)"],
            alpha_2_code="LV",
            alpha_3_code="LVA",
            continent="Европа",
            name="Латвия",
            capital="Рига",
        ),
        Country(
            timezones=["Ливия (UTC+02)"],
            alpha_2_code="LY",
            alpha_3_code="LBY",
            continent="Африка",
            name="Ливия",
            capital="Триполи",
        ),
        Country(
            timezones=["Мадагаскар (UTC+03)"],
            alpha_2_code="MG",
            alpha_3_code="MDG",
            continent="Африка",
            name="Мадагаскар",
            capital="Антананариву",
        ),
        Country(
            timezones=["Маршалловы Острова (UTC+12)"],
            alpha_2_code="MH",
            alpha_3_code="MHL",
            continent="Океания",
            name="Маршалловы Острова",
            capital="Маджуро",
        ),
        Country(
            timezones=["Северная Македония (UTC+01)"],
            alpha_2_code="MK",
            alpha_3_code="MKD",
            continent="Европа",
            name="Северная Македония",
            capital="Скопье",
        ),
        Country(
            timezones=["Мали (UTC)"],
            alpha_2_code="ML",
            alpha_3_code="MLI",
            continent="Африка",
            name="Мали",
            capital="Бамако",
        ),
        Country(
            timezones=["Мьянма (UTC+06:30)"],
            alpha_2_code="MM",
            alpha_3_code="MMR",
            continent="Азия",
            name="Мьянма",
            capital="Нейпьидо",
        ),
        Country(
            timezones=["Монголия (UTC+07)", "Монголия (UTC+08)"],
            alpha_2_code="MN",
            alpha_3_code="MNG",
            continent="Азия",
            name="Монголия",
            capital="Улан-Батор",
        ),
        Country(
            timezones=["Мавритания (UTC)"],
            alpha_2_code="MR",
            alpha_3_code="MRT",
            continent="Африка",
            name="Мавритания",
            capital="Нуакшот",
        ),
        Country(
            timezones=["Мальта (UTC+01)"],
            alpha_2_code="MT",
            alpha_3_code="MLT",
            continent="Европа",
            name="Мальта",
            capital="Валлетта",
        ),
        Country(
            timezones=["Маврикий (UTC+04)"],
            alpha_2_code="MU",
            alpha_3_code="MUS",
            continent="Африка",
            name="Маврикий",
            capital="Порт-Луи",
        ),
        Country(
            timezones=["Мальдивы (UTC+05)"],
            alpha_2_code="MV",
            alpha_3_code="MDV",
            continent="Азия",
            name="Мальдивы",
            capital="Мале",
        ),
        Country(
            timezones=["Малави (UTC+02)"],
            alpha_2_code="MW",
            alpha_3_code="MWI",
            continent="Африка",
            name="Малави",
            capital="Лилонгве",
        ),
        Country(
            timezones=["Мексика (UTC-08)", "Мексика (UTC-07)", "Мексика (UTC-06)"],
            alpha_2_code="MX",
            alpha_3_code="MEX",
            continent="Северная Америка",
            name="Мексика",
            capital="Мехико",
        ),
        Country(
            timezones=["Малайзия (UTC+08)"],
            alpha_2_code="MY",
            alpha_3_code="MYS",
            continent="Азия",
            name="Малайзия",
            capital="Куала-Лумпур",
        ),
        Country(
            timezones=["Мозамбик (UTC+02)"],
            alpha_2_code="MZ",
            alpha_3_code="MOZ",
            continent="Африка",
            name="Мозамбик",
            capital="Мапуту",
        ),
        Country(
            timezones=["Намибия (UTC+01)"],
            alpha_2_code="NA",
            alpha_3_code="NAM",
            continent="Африка",
            name="Намибия",
            capital="Виндхук",
        ),
        Country(
            timezones=["Нигер (UTC+01)"],
            alpha_2_code="NE",
            alpha_3_code="NER",
            continent="Африка",
            name="Нигер",
            capital="Ниамей",
        ),
        Country(
            timezones=["Нигерия (UTC+01)"],
            alpha_2_code="NG",
            alpha_3_code="NGA",
            continent="Африка",
            name="Нигерия",
            capital="Абуджа",
        ),
        Country(
            timezones=["Никарагуа (UTC-06)"],
            alpha_2_code="NI",
            alpha_3_code="NIC",
            continent="Северная Америка",
            name="Никарагуа",
            capital="Манагуа",
        ),
        Country(
            timezones=["Нидерланды (UTC+01)"],
            alpha_2_code="NL",
            alpha_3_code="NLD",
            continent="Европа",
            name="Нидерланды",
            capital="Амстердам",
        ),
        Country(
            timezones=["Норвегия (UTC+01)"],
            alpha_2_code="NO",
            alpha_3_code="NOR",
            continent="Европа",
            name="Норвегия",
            capital="Осло",
        ),
        Country(
            timezones=["Непал (UTC+05:45"],
            alpha_2_code="NP",
            alpha_3_code="NPL",
            continent="Азия",
            name="Непал",
            capital="Катманду",
        ),
        Country(
            timezones=["Науру (UTC+12)"],
            alpha_2_code="NR",
            alpha_3_code="NRU",
            continent="Океания",
            name="Науру",
            capital="Ярен",
        ),
        Country(
            timezones=["Новая Зеландия (UTC+12)"],
            alpha_2_code="NZ",
            alpha_3_code="NZL",
            continent="Океания",
            name="Новая Зеландия",
            capital="Веллингтон",
        ),
        Country(
            timezones=["Оман (UTC+04"],
            alpha_2_code="OM",
            alpha_3_code="OMN",
            continent="Азия",
            name="Оман",
            capital="Маскат",
        ),
        Country(
            timezones=["Панама (UTC-05)"],
            alpha_2_code="PA",
            alpha_3_code="PAN",
            continent="Северная Америка",
            name="Панама",
            capital="Панама",
        ),
        Country(
            timezones=["Перу (UTC-05)"],
            alpha_2_code="PE",
            alpha_3_code="PER",
            continent="Южная Америка",
            name="Перу",
            capital="Лима",
        ),
        Country(
            timezones=["Папуа - Новая Гвинея (UTC+10)"],
            alpha_2_code="PG",
            alpha_3_code="PNG",
            continent="Океания",
            name="Папуа - Новая Гвинея",
            capital="Порт-Морсби",
        ),
        Country(
            timezones=["Филиппины (UTC+08)"],
            alpha_2_code="PH",
            alpha_3_code="PHL",
            continent="Азия",
            name="Филиппины",
            capital="Манила",
        ),
        Country(
            timezones=["Пакистан (UTC+05)"],
            alpha_2_code="PK",
            alpha_3_code="PAK",
            continent="Азия",
            name="Пакистан",
            capital="Исламабад",
        ),
        Country(
            timezones=["Польша (UTC+01)"],
            alpha_2_code="PL",
            alpha_3_code="POL",
            continent="Европа",
            name="Польша",
            capital="Варшава",
        ),
        Country(
            timezones=["Португалия (UTC)"],
            alpha_2_code="PT",
            alpha_3_code="PRT",
            continent="Европа",
            name="Португалия",
            capital="Лиссабон",
        ),
        Country(
            timezones=["Палау (UTC+09)"],
            alpha_2_code="PW",
            alpha_3_code="PLW",
            continent="Океания",
            name="Палау",
            capital="Кампала",
        ),
        Country(
            timezones=["Парагвай (UTC-04)"],
            alpha_2_code="PY",
            alpha_3_code="PRY",
            continent="Южная Америка",
            name="Парагвай",
            capital="Асунсьон",
        ),
        Country(
            timezones=["Катар (UTC+03)"],
            alpha_2_code="QA",
            alpha_3_code="QAT",
            continent="Азия",
            name="Катар",
            capital="Доха",
        ),
        Country(
            timezones=["Румыния (UTC+02)"],
            alpha_2_code="RO",
            alpha_3_code="ROU",
            continent="Европа",
            name="Румыния",
            capital="Бухарест",
        ),
        Country(
            timezones=[
                "Россия (UTC+02)",
                "Россия (UTC+03)",
                "Россия (UTC+04)",
                "Россия (UTC+05)",
                "Россия (UTC+06)",
                "Россия (UTC+07)",
                "Россия (UTC+08)",
                "Россия (UTC+09)",
                "Россия (UTC+10)",
                "Россия (UTC+11)",
                "Россия (UTC+12)",
            ],
            alpha_2_code="RU",
            alpha_3_code="RUS",
            continent="Европа",
            name="Россия",
            capital="Москва",
        ),
        Country(
            timezones=["Руанда (UTC+02)"],
            alpha_2_code="RW",
            alpha_3_code="RWA",
            continent="Африка",
            name="Руанда",
            capital="Кигали",
        ),
        Country(
            timezones=["Саудовская Аравия (UTC+03)"],
            alpha_2_code="SA",
            alpha_3_code="SAU",
            continent="Азия",
            name="Саудовская Аравия",
            capital="Эр-Рияд",
        ),
        Country(
            timezones=["Соломоновы Острова (UTC+11)"],
            alpha_2_code="SB",
            alpha_3_code="SLB",
            continent="Океания",
            name="Соломоновы Острова",
            capital="Хониара",
        ),
        Country(
            timezones=["Сейшельские острова (UTC+04)"],
            alpha_2_code="SC",
            alpha_3_code="SYC",
            continent="Африка",
            name="Сейшельские острова",
            capital="Виктория",
        ),
        Country(
            timezones=["Судан (UTC+03)"],
            alpha_2_code="SD",
            alpha_3_code="SDN",
            continent="Африка",
            name="Судан",
            capital="Хартум",
        ),
        Country(
            timezones=["Швеция (UTC+01)"],
            alpha_2_code="SE",
            alpha_3_code="SWE",
            continent="Европа",
            name="Швеци",
            capital="Стокгольм",
        ),
        Country(
            timezones=["Сингапур (UTC+08)"],
            alpha_2_code="SG",
            alpha_3_code="SGP",
            continent="Азия",
            name="Сингапур",
            capital="Сингапур",
        ),
        Country(
            timezones=["Словения (UTC+01)"],
            alpha_2_code="SI",
            alpha_3_code="SVN",
            continent="Европа",
            name="Словения",
            capital="Любляна",
        ),
        Country(
            timezones=["Словакия (UTC+01)"],
            alpha_2_code="SK",
            alpha_3_code="SVK",
            continent="Европа",
            name="Словакия",
            capital="Братислава",
        ),
        Country(
            timezones=["Сьерра-Леоне (UTC)"],
            alpha_2_code="SL",
            alpha_3_code="SLE",
            continent="Африка",
            name="Сьерра Леоне",
            capital="Фритаун",
        ),
        Country(
            timezones=["Сан-Марино (UTC+01)"],
            alpha_2_code="SM",
            alpha_3_code="SMR",
            continent="Европа",
            name="Сан-Марино",
            capital="Сан-Марино",
        ),
        Country(
            timezones=["Сенегал (UTC)"],
            alpha_2_code="SN",
            alpha_3_code="SEN",
            continent="Африка",
            name="Сенегал",
            capital="Дакар",
        ),
        Country(
            timezones=["Сомали (UTC+03)"],
            alpha_2_code="SO",
            alpha_3_code="SOM",
            continent="Африка",
            name="Сомали",
            capital="Могадишо",
        ),
        Country(
            timezones=["Суринам (UTC-03)"],
            alpha_2_code="SR",
            alpha_3_code="SUR",
            continent="Южная Америка",
            name="Суринам",
            capital="Парамарибо",
        ),
        Country(
            timezones=["Сан-Томе и Принсипи (UTC)"],
            alpha_2_code="ST",
            alpha_3_code="STP",
            continent="Африка",
            name="Сан-Томе и Принсипи",
            capital="Сан-Томе",
        ),
        Country(
            timezones=["Сирия (UTC+02)"],
            alpha_2_code="SY",
            alpha_3_code="SYR",
            continent="Азия",
            name="Сирия",
            capital="Дамаск",
        ),
        Country(
            timezones=["Того (UTC)"],
            alpha_2_code="TG",
            alpha_3_code="TGO",
            continent="Африка",
            name="Того",
            capital="Ломе",
        ),
        Country(
            timezones=["Таиланд (UTC+07)"],
            alpha_2_code="TH",
            alpha_3_code="THA",
            continent="Азия",
            name="Таиланд",
            capital="Бангкок",
        ),
        Country(
            timezones=["Таджикистан (UTC+05)"],
            alpha_2_code="TJ",
            alpha_3_code="TJK",
            continent="Азия",
            name="Таджикистан",
            capital="Душанбе",
        ),
        Country(
            timezones=["Туркмения (UTC+05)"],
            alpha_2_code="TM",
            alpha_3_code="TKM",
            continent="Азия",
            name="Туркмения",
            capital="Ашхабад",
        ),
        Country(
            timezones=["Тунис (UTC+01)"],
            alpha_2_code="TN",
            alpha_3_code="TUN",
            continent="Африка",
            name="Тунис",
            capital="Тунис",
        ),
        Country(
            timezones=["Тонга (UTC+13)"],
            alpha_2_code="TO",
            alpha_3_code="TON",
            continent="Океания",
            name="Тонга",
            capital="Нукуалофа",
        ),
        Country(
            timezones=["Турция (UTC+02)"],
            alpha_2_code="TR",
            alpha_3_code="TUR",
            continent="Азия",
            name="Турция",
            capital="Анкара",
        ),
        Country(
            timezones=["Тринидад и Тобаго (UTC-04)"],
            alpha_2_code="TT",
            alpha_3_code="TTO",
            continent="Северная Америка",
            name="Тринидад и Тобаго",
            capital="Порт-оф-Спейн",
        ),
        Country(
            timezones=["Тувалу (UTC+12)"],
            alpha_2_code="TV",
            alpha_3_code="TUV",
            continent="Океания",
            name="Тувалу",
            capital="Фунафути",
        ),
        Country(
            timezones=["Танзания (UTC+03)"],
            alpha_2_code="TZ",
            alpha_3_code="TZA",
            continent="Африка",
            name="Танзания",
            capital="Додома",
        ),
        Country(
            timezones=["Украина (UTC+02)", "Украина (UTC+03)"],
            alpha_2_code="UA",
            alpha_3_code="UKR",
            continent="Европа",
            name="Украина",
            capital="Киев",
        ),
        Country(
            timezones=["Уганда (UTC+03)"],
            alpha_2_code="UG",
            alpha_3_code="UGA",
            continent="Африка",
            name="Уганда",
            capital="Кампала",
        ),
        Country(
            timezones=[
                "США (UTC-11)",
                "США (UTC-10)",
                "США (UTC-09)",
                "США (UTC-08)",
                "США (UTC-07)",
                "США (UTC-06)",
                "США (UTC-05)",
                "США (UTC-04)",
                "США (UTC+10)",
            ],
            alpha_2_code="US",
            alpha_3_code="USA",
            continent="Северная Америка",
            name="США",
            capital="Вашингтон",
        ),
        Country(
            timezones=["Уругвай (UTC-03)"],
            alpha_2_code="UY",
            alpha_3_code="URY",
            continent="Южная Америка",
            name="Уругвай",
            capital="Монтевидео",
        ),
        Country(
            timezones=["Узбекистан (UTC+05)"],
            alpha_2_code="UZ",
            alpha_3_code="UZB",
            continent="Азия",
            name="Узбекистан",
            capital="Ташкент",
        ),
        Country(
            timezones=["Ватикан (UTC+01)"],
            alpha_2_code="VA",
            alpha_3_code="VAT",
            continent="Европа",
            name="Ватикан",
            capital="Ватикан",
        ),
        Country(
            timezones=["Венесуэла (UTC-04:30)"],
            alpha_2_code="VE",
            alpha_3_code="VEN",
            continent="Южная Америка",
            name="Венесуэла",
            capital="Каракас",
        ),
        Country(
            timezones=["Вьетнам (UTC+07)"],
            alpha_2_code="VN",
            alpha_3_code="VNM",
            continent="Азия",
            name="Вьетнам",
            capital="Ханой",
        ),
        Country(
            timezones=["Вануату (UTC+11)"],
            alpha_2_code="VU",
            alpha_3_code="VUT",
            continent="Океания",
            name="Вануату",
            capital="Порт-Вила",
        ),
        Country(
            timezones=["Йемен (UTC+03)"],
            alpha_2_code="YE",
            alpha_3_code="YEM",
            continent="Азия",
            name="Йемен",
            capital="Сана",
        ),
        Country(
            timezones=["Замбия (UTC+02)"],
            alpha_2_code="ZM",
            alpha_3_code="ZMB",
            continent="Африка",
            name="Замбия",
            capital="Лусака",
        ),
        Country(
            timezones=["Зимбабве (UTC+02)"],
            alpha_2_code="ZW",
            alpha_3_code="ZWE",
            continent="Африка",
            name="Зимбабве",
            capital="Хараре",
        ),
        Country(
            timezones=["Алжир (UTC+01)"],
            alpha_2_code="DZ",
            alpha_3_code="DZA",
            continent="Африка",
            name="Алжир",
            capital="Алжир",
        ),
        Country(
            timezones=["Босния и Герцеговина (UTC+01)"],
            alpha_2_code="BA",
            alpha_3_code="BIH",
            continent="Европа",
            name="Босния и Герцеговина",
            capital="Сараево",
        ),
        Country(
            timezones=["Камбоджа (UTC+07)"],
            alpha_2_code="KH",
            alpha_3_code="KHM",
            continent="Азия",
            name="Камбоджа",
            capital="Пномпень",
        ),
        Country(
            timezones=["ЦАР (UTC+01)"],
            alpha_2_code="CF",
            alpha_3_code="CAF",
            continent="Африка",
            name="ЦАР",
            capital="Банги",
        ),
        Country(
            timezones=["Чад (UTC+01)"],
            alpha_2_code="TD",
            alpha_3_code="TCD",
            continent="Африка",
            name="Чад",
            capital="Нджамена",
        ),
        Country(
            timezones=["Коморы (UTC+03)"],
            alpha_2_code="KM",
            alpha_3_code="COM",
            continent="Африка",
            name="Коморы",
            capital="Морони",
        ),
        Country(
            timezones=["Хорватия (UTC+01)"],
            alpha_2_code="HR",
            alpha_3_code="HRV",
            continent="Европа",
            name="Хорватия",
            capital="Загреб",
        ),
        Country(
            timezones=["Восточный Тимор (UTC+09)"],
            alpha_2_code="TL",
            alpha_3_code="TLS",
            continent="Азия",
            name="Восточный Тимор",
            capital="Дили",
        ),
        Country(
            timezones=["Сальвадор (UTC-06)"],
            alpha_2_code="SV",
            alpha_3_code="SLV",
            continent="Северная Америка",
            name="Сальвадор",
            capital="Сан-Сальвадор",
        ),
        Country(
            timezones=["Экваториальная Гвинея (UTC+01)"],
            alpha_2_code="GQ",
            alpha_3_code="GNQ",
            continent="Африка",
            name="Экваториальная Гвинея",
            capital="Малабо",
        ),
        Country(
            timezones=["Гренада (UTC-04)"],
            alpha_2_code="GD",
            alpha_3_code="GRD",
            continent="Северная Америка",
            name="Гренада",
            capital="Сент-Джорджес",
        ),
        Country(
            timezones=["Казахстан (UTC+05)", "Казахстан (UTC+06)"],
            alpha_2_code="KZ",
            alpha_3_code="KAZ",
            continent="Азия",
            name="Казахстан",
            capital="Нур-Султан (Астана)",
        ),
        Country(
            timezones=["Лаос (UTC+07)"],
            alpha_2_code="LA",
            alpha_3_code="LAO",
            continent="Азия",
            name="Лаос",
            capital="Вьентьян",
        ),
        Country(
            timezones=["Микронезия (UTC+10)", "Микронезия (UTC+11)"],
            alpha_2_code="FM",
            alpha_3_code="FSM",
            continent="Океания",
            name="Микронезия",
            capital="Паликир",
        ),
        Country(
            timezones=["Молдавия (UTC+02)"],
            alpha_2_code="MD",
            alpha_3_code="MDA",
            continent="Европа",
            name="Молдавия",
            capital="Кишинев",
        ),
        Country(
            timezones=["Монако (UTC+01)"],
            alpha_2_code="MC",
            alpha_3_code="MCO",
            continent="Европа",
            name="Монако",
            capital="Монако",
        ),
        Country(
            timezones=["Черногория (UTC+01)"],
            alpha_2_code="ME",
            alpha_3_code="MNE",
            continent="Европа",
            name="Черногория",
            capital="Подгорица",
        ),
        Country(
            timezones=["Марокко (UTC)"],
            alpha_2_code="MA",
            alpha_3_code="MAR",
            continent="Африка",
            name="Марокко",
            capital="Рабат",
        ),
        Country(
            timezones=["Сент-Китс и Невис (UTC-04)"],
            alpha_2_code="KN",
            alpha_3_code="KNA",
            continent="Северная Америка",
            name="Сент-Китс и Невис",
            capital="Бастер",
        ),
        Country(
            timezones=["Сент-Люсия (UTC-04)"],
            alpha_2_code="LC",
            alpha_3_code="LCA",
            continent="Северная Америка",
            name="Сент-Люсия",
            capital="Кастри",
        ),
        Country(
            timezones=["Сент-Винсент и Гренадины (UTC-04)"],
            alpha_2_code="VC",
            alpha_3_code="VCT",
            continent="Северная Америка",
            name="Сент-Винсент и Гренадины",
            capital="Кингстаун",
        ),
        Country(
            timezones=["Самоа (UTC+13)"],
            alpha_2_code="WS",
            alpha_3_code="WSM",
            continent="Океания",
            name="Самоа",
            capital="Апиа",
        ),
        Country(
            timezones=["Сербия (UTC+01)"],
            alpha_2_code="RS",
            alpha_3_code="SRB",
            continent="Европа",
            name="Сербия",
            capital="Белград",
        ),
        Country(
            timezones=["ЮАР (UTC+02)"],
            alpha_2_code="ZA",
            alpha_3_code="ZAF",
            continent="Африка",
            name="ЮАР",
            capital="Претория",
        ),
        Country(
            timezones=["Испания (UTC)", "Испания (UTC+01)"],
            alpha_2_code="ES",
            alpha_3_code="ESP",
            continent="Европа",
            name="Испания",
            capital="Мадрид",
        ),
        Country(
            timezones=["Шри-Ланка (UTC+05:30)"],
            alpha_2_code="LK",
            alpha_3_code="LKA",
            continent="Азия",
            name="Шри-Ланка",
            capital="Шри-Джаяварденепура-Котте",
        ),
        Country(
            timezones=["Эсватини (Свазиленд) (UTC+02)"],
            alpha_2_code="SZ",
            alpha_3_code="SWZ",
            continent="Африка",
            name="Эсватини (Свазиленд)",
            capital="Мбабане",
        ),
        Country(
            timezones=["Швейцария (UTC+01)"],
            alpha_2_code="CH",
            alpha_3_code="CHE",
            continent="Европа",
            name="Швейцария",
            capital="Берн",
        ),
        Country(
            timezones=["ОАЭ (UTC+04)"],
            alpha_2_code="AE",
            alpha_3_code="ARE",
            continent="Азия",
            name="ОАЭ",
            capital="Абу-Даби",
        ),
        Country(
            timezones=["Великобритания (UTC)"],
            alpha_2_code="GB",
            alpha_3_code="GBR",
            continent="Европа",
            name="Великобритания",
            capital="Лондон",
        ),
    ]

    def day_of_week(self) -> str:
        day = self.date("%w")
        return self.DAY_NAMES[day]

    def month_name(self) -> str:
        month = self.month()
        return self.MONTH_NAMES[month]



from .. import Provider as PersonProvider


class Provider(PersonProvider):
    formats = ("{{first_name}} {{last_name}}",)

    # First names are from
    #   https://mummyname.net/pakistani-boy-names/
    # Last names are from https://mummyname.net/pakistani-boy-names/

    first_names = (
        "Ali",
        "Arham",
        "Aryan",
        "Ayaan",
        "Faizan",
        "Hamza",
        "Huzaifa",
        "Rayan",
        "Rohaan",
        "Atfat",
        "Atheel",
        "Attaf",
        "Auraq",
        "Awadil",
        "Awamil",
        "Awamiri",
        "Awan",
        "Awani",
        "Awj",
        "Awlya",
        "Awmar",
        "Awrad",
        "Ayamin",
        "Aysar",
        "Ayyubi",
        "Azban",
        "Azeeb",
        "Baashir",
        "Badawi",
        "Bairbel",
        "Bambad",
        "Berezat",
        "Birousk",
        "Bizhan",
        "Buraid",
        "Chamali",
        "Changaz",
        "Charlesh",
        "Chashida",
        "Chawish",
        "Cheragh",
        "Darain",
        "Dastageer",
        "Dayyaan",
        "Durab",
        "Ejlaal",
        "Elaf",
        "Esfandyar",
        "Etizaaz",
        "Fahmi",
        "Fazli",
        "Fidvi",
        "Ghazanfer",
        "Gulfaam",
        "Guney",
        "Harnail",
        "Hazeem",
        "Kachela",
        "Khaan",
        "Mizhir",
        "Mourib",
        "Muhallil",
        "Muhazzim",
        "Muzdahir",
        "Muzhir",
        "Rusul",
        "Ruwaihim",
        "Samama",
        "Souma",
        "Tadeen",
        "Tafazal",
        "Tahmaseb",
        "Tahoor",
        "Taial",
        "Taisir",
        "Tarfaan",
        "Tawkeel",
        "Tirdad",
        "Tishk",
        "Urrab",
        "Vahar",
        "Xobeen",
        "Yadid",
        "Yafir",
        "Yamar",
        "Yashem",
        "Yazan",
        "Yazeed",
        "Yeraz",
        "Yergha",
        "Yesoob",
        "Yureed",
        "Zabir",
        "Zahaar",
        "Zahri",
        "Zahrun",
        "Zahur",
        "Zarar",
        "Zauqi",
        "Zaweel",
        "Zayan",
        "Zayyir",
        "Zerdad",
        "Zewad",
        "Zimran",
        "Zuwayhir",
        "Adil",
        "Aaaqil",
        "Aaban",
        "Aabid",
        "Aabid",
        "Aadam",
        "Aadil",
        "Aadil",
        "Aadil",
        "Aafiya",
        "Aahil",
        "Aalam",
        "Aalam",
        "Aalee",
        "Aalim",
        "Aamil",
        "Aamir",
        "Aamir",
        "Aamir",
        "Aamirah",
        "Aaqil",
        "Aarif",
        "Aarif",
        "Aarif",
        "Aariz",
        "Aaryan",
        "Aashif",
        "Aashir",
        "Aasif",
        "Aasif",
        "Aasim",
        "Aasim",
        "Aasim",
        "Aatif",
        "Aatiq",
        "Aaus",
        "Aayan",
        "Aazim",
        "Abaan",
        "Baahir",
        "Baaizeed",
        "Baaqee",
        "Baaqir",
        "Baaree",
        "Baasim",
        "Baasit",
        "Babar",
        "Baber",
        "Badr",
        "Badr Udeen",
        "Baha",
        "Baha Udeen",
        "Bahaa",
        "Bahiy Udeen",
        "Baleegh",
        "Baqar",
        "Baqir",
        "Barr",
        "Barraq",
        "Basaam",
        "Baseer",
        "Basel",
        "Basem",
        "Bashaar",
        "Bashaarat",
        "Bashar",
        "Basharat",
        "Basheer",
        "Basheerah",
        "Basil",
        "Basim",
        "Basir",
        "Bassam",
        "Batal",
        "Batool",
        "Bazam",
        "Bilaal",
        "Bilal",
        "Dawud",
        "Daamin",
        "Daanish",
        "Daanyaal",
        "Daawood",
        "Dabbah",
        "Dabir",
        "Daghfal",
        "Daiyaan",
        "Dakhil",
        "Dameer",
        "Damurah",
        "Daniel",
        "Danish",
        "Eesaa",
        "Ehan",
        "Ehsaas",
        "Ehsan",
        "Eijaz",
        "Ejaz",
        "El-Amin",
        "Emran",
        "Eshan",
        "Ghaalib",
        "Ghaazi",
        "Ghaffaar",
        "Ghafoor",
        "Ghaith",
        "Ghalib",
        "Ghanee",
        "Ghanem",
        "Ghannam",
        "Ghasaan",
        "Ghauth",
        "Ghauth",
        "Ghawth",
        "Ghayoor",
        "Ghazalan",
        "Ghazanfar",
        "Ghazawan",
        "Ghazi",
        "Ghazzal",
        "Ghiyaath",
        "Ghiyath",
        "Ghufran",
        "Ghulaam",
        "Ghulam",
        "Ghunayn",
        "Ghusharib",
        "Ghusun",
        "Ghutayf",
        "Gohar",
        "Gulab",
        "Gulfam",
        "Gulshan",
        "Gulzar",
        "Izaaz",
        "Ibaad",
        "Ibn",
        "Ibraaheem",
        "Ibraheem",
        "Ibrahim",
        "Idrees",
        "Idrees",
        "Idris",
        "Iesa",
        "Iftikhaar",
        "Iftikhar",
        "Ihab",
        "Ihsaan",
        "Ihsaan",
        "Ihsan",
        "Ihtesham",
        "Ihtiram",
        "Ihtishaam",
        "Ihtsham",
        "Ijli",
        "Ikhlaas",
        "Ikraam",
        "Ikramah",
        "Ikrimah",
        "Ikrimah",
        "Ilan",
        "Jafar",
        "Jaabir",
        "Jaabir",
        "Jaafar",
        "Jaan",
        "Jabbaar",
        "Jabir",
        "Jabr",
        "Jad",
        "Jafar",
        "Jaffer",
        "Jahangir",
        "Jahanzeb",
        "Jahdami",
        "Jahdari",
        "Jahiz",
        "Jahm",
        "Jalaal",
        "Jalaal",
        "Jalal",
        "Jalees",
        "Jalil",
        "Jamaal",
        "Jamaal",
        "Jamaal Udeen",
        "Jamal",
        "Jameel",
        "Jameel",
        "Kaamil",
        "Kaamil",
        "Kaamil",
        "Kaashif",
        "Kaazim",
        "Kabeer",
        "Kabeer",
        "Kafeel",
        "Kaiser",
        "Kajji",
        "Kalbi",
        "Kaleem",
        "Kaleem",
        "Kaleema",
        "Kamal",
        "Kamal",
        "Kamil",
        "Kamran",
        "Karaamat",
        "Karam",
        "Kareem",
        "Kareem",
        "Karim",
        "Kasam",
        "Kashan",
        "Kashif",
        "Kasim",
        "Kauthar",
        "Kawkab",
        "Kawthar",
        "Kaysan",
        "Kazi",
        "Kazim",
        "Keyaan",
        "Khaalid",
        "Laeeq",
        "Labeeb",
        "Labeeb",
        "Labib",
        "Lahiah",
        "Laiq",
        "Laith",
        "Lajlaj",
        "Laqeet",
        "Lateef",
        "Lateef",
        "Latif",
        "Layth",
        "Liban",
        "Limazah",
        "Liyaaqat",
        "Liyaqat",
        "Loot",
        "Luay",
        "Luqmaan",
        "Luqmaan",
        "Luqman",
        "Lut",
        "Lutf",
        "Lutf",
        "Lutfi",
        "Lutfi",
        "Maawiya",
        "Mad",
        "Mamun",
        "Man",
        "Man",
        "Maroof",
        "Maahir",
        "Maajid",
        "Maalik",
        "Maaz",
        "Maazin",
        "Mabad",
        "Madani",
        "Madiyan",
        "Madyan",
        "Mahad",
        "Mahaz",
        "Mahbeer",
        "Mahboob",
        "Mahbub",
        "Mahdee",
        "Mahdi",
        "Mahdy",
        "Maheen",
        "Maher",
        "Mahfooz",
        "Mahfuj",
        "Mahfuz",
        "Mahja",
        "Mahmood",
        "Mahmoud",
        "Mahmud",
        "Majd",
        "Majd",
        "Majd Udeen",
        "Majdi",
        "Majdy",
        "Majeed",
        "Makeen",
        "Nail",
        "Naail",
        "Naadir",
        "Naadir",
        "Naajy",
        "Naasih",
        "Naasir",
        "Naathim",
        "Naazhim",
        "Nabeeh",
        "Nabeel",
        "Nabeel",
        "Nabeel",
        "Nabhan",
        "Nabhan",
        "Nabigh",
        "Nabih",
        "Nabil",
        "Nadeem",
        "Nadeem",
        "Nadhir",
        "Nadim",
        "Nadir",
        "Nadr",
        "Naeem",
        "Naeem",
        "Nafasat",
        "Nafees",
        "Nafees",
        "Nafesa",
        "Nafis",
        "Naib",
        "Naim",
        "Najair",
        "Najam",
        "Najam",
        "Najeeb",
        "Najeeb",
        "Najeeb",
        "Obaid",
        "Omair",
        "Omar",
        "Omar",
        "Omeir",
        "Omran",
        "Osama",
        "Ossama",
        "Owais",
        "Parsa",
        "Parvez",
        "Pervaiz",
        "Qaadir",
        "Qaadir",
        "Qaasim",
        "Qabeel",
        "Qadar",
        "Qadeer",
        "Qadeer",
        "Qadi",
        "Qadim",
        "Qahtan",
        "Qaim",
        "Qais",
        "Qamar",
        "Qani",
        "Qanit",
        "Qareeb",
        "Qaseem",
        "Qasid",
        "Qasif",
        "Qasim",
        "Qatadah",
        "Qatadah",
        "Qawee",
        "Qawee",
        "Qay-yoom",
        "Qays",
        "Quadir",
        "Qudamah",
        "Qudamah",
        "Quddoos",
        "Qudoos",
        "Qurban",
        "Qusay",
        "Qutaybah",
        "Qutaybah",
        "Qutb",
        "Qutub",
        "Raed",
        "Raid",
        "Raaghib",
        "Raahil",
        "Raakin",
        "Raamis",
        "Raamiz",
        "Raamiz",
        "Raashid",
        "Raashid",
        "Raatib",
        "Rabah",
        "Rabah",
        "Rabar",
        "Rabb",
        "Rabbaanee",
        "Rabbani",
        "Rabee",
        "Rabee",
        "Rabiah",
        "Rabit",
        "Radhee",
        "Radi",
        "Raees",
        "Rafan",
        "Rafay",
        "Rafee",
        "Rafee",
        "Rafeek",
        "Rafeeq",
        "Rafi",
        "Rafiq",
        "Ragheb",
        "Raghib",
        "Rahat",
        "Raheel",
        "Raheem",
        "Sad",
        "Sadan",
        "Said",
        "Saim",
        "Sair",
        "Sairah",
        "Saood",
        "Saabir",
        "Saabir",
        "Saad",
        "Saad",
        "Saadat",
        "Saadiq",
        "Saafir",
        "Saahir",
        "Saahir",
        "Saaiq",
        "Saajid",
        "Saajid",
        "Saal",
        "Saalih",
        "Saalim",
        "Saaqib",
        "Saariyah",
        "Sabah",
        "Sabahat",
        "Sabbir",
        "Sabeeh",
        "Sabeeh",
        "Sabeeh",
        "Sabil",
        "Sabiq",
        "Sabir",
        "Saboor",
        "Sabur",
        "Saburah",
        "Sadan",
        "Sadaqat",
        "Sadeed",
        "Taahaa",
        "Taahir",
        "Taahir",
        "Taaj",
        "Taalib",
        "Taamir",
        "Taanish",
        "Taariq",
        "Taban",
        "Tabassum",
        "Tabassum",
        "Tabish",
        "Taha",
        "Tahaw-wur",
        "Tahawwur",
        "Taheem",
        "Tahir",
        "Tahmeed",
        "Tahmid",
        "Tahseen",
        "Taimur",
        "Taj",
        "Tajammal",
        "Tajammul",
        "Tajammul",
        "Tajudinn",
        "Talal",
        "Talal",
        "Talat",
        "Talha",
        "Talha",
        "Talhah",
        "Talhah",
        "Tali",
        "Talib",
        "Tamam",
        "Tamanna",
        "Ubaadah",
        "Ubaadah",
        "Ubaadah",
        "Ubaadah",
        "Ubadah",
        "Ubadah",
        "Ubaid",
        "Ubaid",
        "Ubaidah",
        "Ubaidah",
        "Ubay",
        "Ubay",
        "Ubayd",
        "Ubayd",
        "Ubaydullah",
        "Ubaydullah",
        "Uhban",
        "Uhban",
        "Ulfat",
        "Ulfat",
        "Ulfat",
        "Umaarah",
        "Umaarah",
        "Umair",
        "Umair",
        "Umair",
        "Umair",
        "Umar",
        "Umar",
        "Umar",
        "Umar",
        "Umar",
        "Waail",
        "Waail",
        "Waahid",
        "Waahid",
        "Waajid",
        "Wadee",
        "Wadi",
        "Wadood",
        "Wafa",
        "Wafeeq",
        "Wafi",
        "Wafiq",
        "Wahab",
        "Wahb",
        "Wahban",
        "Waheed",
        "Waheed",
        "Wahhaab",
        "Wahhaaj",
        "Yaaseen",
        "Yafi",
        "Yaghnam",
        "Yahya",
        "Yahyaa",
        "Yaman",
        "Yaman",
        "Yameen",
        "Yaqeen",
        "Yaqoot",
        "Yaqub",
        "Yar",
        "Yasaar",
        "Yaseen",
        "Yasin",
        "Yasir",
        "Yasir",
        "Yathrib",
        "Yawar",
        "Yawer",
        "Zaafir",
        "Zaahid",
        "Zaahid",
        "Zaahir",
        "Zaahir",
        "Zaahir",
        "Zaakir",
        "Zackariya",
        "Zaeem",
        "Zafar",
        "Zafar",
        "Zafeer",
        "Zafir",
        "Zafrul",
        "Zaheer",
        "Zaheer",
        "Zaheer",
        "Zahi",
        "Zahir",
        "Zaib",
    )

    last_names = (
        "Lajlaj",
        "Aarif",
        "Urrab",
        "Tabassum",
        "Ubadah",
        "Daniel",
        "Umaarah",
        "Omair",
        "Jalil",
        "Aatiq",
        "Karaamat",
        "Lut",
        "Karam",
        "Aasif",
        "Aadam",
        "Mahbeer",
        "Saalim",
        "Ubayd",
        "Naail",
        "Mahfuz",
        "Ghazzal",
        "Aamir",
        "Ubaydullah",
        "Umaarah",
        "Rabiah",
        "Maawiya",
        "Yasir",
        "Raaghib",
        "Daamin",
        "Rabb",
        "Bashaar",
        "Taanish",
        "Yafir",
        "Baaree",
        "Talib",
        "Rafi",
        "Luqman",
        "Qaasim",
        "Ubaidah",
        "Saajid",
        "Yaman",
        "Ubaadah",
        "Baaqir",
        "Sadan",
        "Zarar",
        "Saafir",
        "Zafar",
        "Mahmoud",
        "Zayyir",
        "Ubay",
        "Fidvi",
        "Mahfuj",
        "Awmar",
        "Yawer",
        "Ayaan",
        "Taimur",
        "Rabbani",
        "Ayyubi",
        "Waahid",
        "Ijli",
        "Baleegh",
        "Bilaal",
        "Radi",
        "Ali",
        "Tadeen",
        "Souma",
        "Layth",
        "Kashif",
        "Labeeb",
        "Talhah",
        "Sabir",
        "Dabir",
        "Yaghnam",
        "Zackariya",
        "Ibrahim",
        "Rafeek",
        "Qadeer",
        "Luqmaan",
        "Jahdari",
        "Qabeel",
        "Kaamil",
        "Ilan",
        "Omeir",
        "Ubaid",
        "Majd",
        "Aadil",
        "Ghafoor",
        "Zahrun",
        "Tabassum",
        "Lutf",
        "Aamir",
        "Iftikhaar",
        "Naeem",
        "Ghauth",
        "Eshan",
        "Raid",
        "Qasif",
        "Ihsaan",
        "Bambad",
        "Aaaqil",
        "Nabeel",
        "Jamaal",
        "Awj",
        "Wahhaaj",
        "Nabih",
        "Jalaal",
        "Yahyaa",
        "Aalam",
        "Ghayoor",
        "Aarif",
        "Tahir",
        "Batal",
        "Talha",
        "Uhban",
        "Aryan",
        "Najam",
        "Darain",
        "Qusay",
        "Vahar",
        "Aabid",
        "Ihtiram",
        "Umar",
        "Mahbub",
        "Qaim",
        "Saajid",
        "Owais",
        "Maheen",
        "Raashid",
        "Limazah",
        "Zaafir",
        "Wadood",
        "Aariz",
        "Aalam",
        "Ihab",
        "Umair",
        "Zahri",
        "Aazim",
        "Jad",
        "Omar",
        "Majeed",
        "Qaseem",
        "Rafay",
        "Ghanee",
        "Gulshan",
        "Babar",
        "Baasim",
        "Ghunayn",
        "Jaabir",
        "Nadeem",
        "Lahiah",
        "Sair",
        "Saaqib",
        "Esfandyar",
        "Zaheer",
        "Sabil",
        "Qutaybah",
        "Azban",
        "Zafrul",
        "Awani",
        "Tajammul",
        "Auraq",
        "Man",
        "Tafazal",
        "Raed",
        "Baseer",
        "Quadir",
        "Dawud",
        "Talal",
        "Sabah",
        "Baashir",
        "Damurah",
        "Ibraaheem",
        "Faizan",
        "Zaakir",
        "Ghutayf",
        "Ehsaas",
        "Sadeed",
        "Mad",
        "Jabir",
        "Mourib",
        "Aamil",
        "Sabeeh",
        "Bizhan",
        "Barr",
        "Basaam",
        "Ghasaan",
        "Nail",
        "Kasim",
        "Taaj",
        "Omran",
        "Madiyan",
        "Taheem",
        "Saad",
        "Kamal",
        "Raatib",
        "Taj",
        "Yadid",
        "Basheerah",
        "Aasim",
        "Zahur",
        "Saabir",
        "Kasam",
        "Naeem",
        "Tawkeel",
        "Ghannam",
        "Tahmaseb",
        "Awadil",
        "Liyaaqat",
        "Tahaw-wur",
        "Tamanna",
        "Zafir",
        "Ghauth",
        "Ubay",
        "Zaahid",
        "Awamil",
        "Talat",
        "Maalik",
        "Qadar",
        "Waajid",
        "Aamirah",
        "Ayamin",
        "Kamran",
        "Kaleem",
        "Wadi",
        "Zaahid",
        "Umar",
        "Bashaarat",
        "Saal",
        "Najeeb",
        "Kachela",
        "Sabur",
        "Buraid",
        "Rabee",
        "Najeeb",
        "Yar",
        "Umar",
        "Ossama",
        "Tahawwur",
        "Zaahir",
        "Raashid",
        "Tali",
        "Batool",
        "Umair",
        "Ihsaan",
        "Majd Udeen",
        "Kaamil",
        "Raheel",
        "Abaan",
        "Rabah",
        "Jameel",
        "Gohar",
        "Aabid",
        "Zuwayhir",
        "Sadan",
        "Idris",
        "Qais",
        "Sadaqat",
        "Barraq",
        "Ejlaal",
        "Luay",
        "Jahdami",
        "Wafeeq",
        "Wafa",
        "Rabar",
        "Aasif",
        "Dakhil",
        "Jalaal",
        "Gulfam",
        "Saahir",
        "Maroof",
        "Baasit",
        "Kabeer",
        "Jameel",
        "Latif",
        "Badr Udeen",
        "Qahtan",
        "Liyaqat",
        "Jabr",
        "Kaleema",
        "Fazli",
        "Huzaifa",
        "Man",
        "Rohaan",
        "Ubadah",
        "Saburah",
        "Saariyah",
        "Kaysan",
        "Raakin",
        "Sabiq",
        "Saboor",
        "Zahaar",
        "Jaabir",
    )



from .. import Provider as PersonProvider


def translate_to_bengali_digits(en_digit: str = "0") -> str:
    """
    Translate any English string containing digits to corresponding Bengali digits.
    :example: '9786' to '৯৭৮৬'
    """
    english_to_bengali_digits_map = {
        "0": "০",
        "1": "১",
        "2": "২",
        "3": "৩",
        "4": "৪",
        "5": "৫",
        "6": "৬",
        "7": "৭",
        "8": "৮",
        "9": "৯",
    }
    bn_digit = ""
    for char in en_digit:
        bn_digit = bn_digit + english_to_bengali_digits_map.get(char, char)
    return bn_digit


class Provider(PersonProvider):
    """Implement person provider for ``bn_BD`` locale."""

    prefixes = (
        "ইঞ্জিঃ",
        "ডাঃ",
        "ডঃ",
    )

    prefixes_male = (
        "জনাব",
        "মিঃ",
        "মৃতঃ",
    ) + prefixes

    prefixes_female = (
        "জনাবা",
        "মিসঃ",
        "মিসেস",
        "মৃতাঃ",
    ) + prefixes

    suffixes = (
        "অবঃ",
        "এমএসসি",
        "এমডি",
        "ডিডিএস",
        "ডিভিএম",
        "পিএইচডি",
        "বিএসসি",
    )

    language_names = (
        "আফার",
        "আবখাজিয়ান",
        "আবেস্তান",
        "আফ্রিকান",
        "আকান",
        "আমহারিক",
        "আরাগোনিজ",
        "আরবি",
        "অসমীয়া",
        "অ্যাভারিক",
        "আয়মারা",
        "আজারবাইজানীয়",
        "বাশকির",
        "বেলারুশিয়ান",
        "বুলগেরিয়ান",
        "বিহারী ভাষা" "বিসলামা",
        "বামবারা",
        "বাংলা",
        "তিব্বতি",
        "ব্রেটন",
        "বসনীয়",
        "কাতালান",
        "চেচেন",
        "চামোরো",
        "করসিকান",
        "ক্রি",
        "চেক",
        "চার্চ স্লাভিক",
        "চুভাশ",
        "ওয়েলশ",
        "ড্যানিশ",
        "জার্মান",
        "দিভেহি",
        "জংখা",
        "ইউ",
        "গ্রীক",
        "ইংরেজি",
        "এসপেরান্তো",
        "স্পেনীয়",
        "এস্তোনিয়ান",
        "বাস্ক",
        "ফারসি",
        "ফুলাহ",
        "ফিনিশ",
        "ফিজিয়ান",
        "ফেরোজ",
        "ফরাসি",
        "পশ্চিম ফ্রিসিয়ান",
        "আইরিশ",
        "গেলিক",
        "গ্যালিশিয়ান",
        "গুয়ারানি",
        "গুজরাটি",
        "ম্যানক্স",
        "হাউসা",
        "হিব্রু",
        "হিন্দি",
        "হিরি মোটু",
        "ক্রোয়েশিয়ান",
        "হাইতিয়ান",
        "হাঙ্গেরিয়ান",
        "আর্মেনিয়ান",
        "হেরো",
        "ইন্টারলিঙ্গুয়া",
        "ইন্দোনেশিয়ান",
        "আন্তর্ভাষা",
        "ইগবো",
        "সিচুয়ান ই",
        "ইনুপিয়াক",
        "আমি করি",
        "আইসল্যান্ডিক",
        "ইতালীয়",
        "ইনুকটিটুট",
        "জাপানি",
        "জাভানিজ",
        "জর্জিয়ান",
        "কঙ্গো",
        "কিকুয়ু",
        "কুয়ানিয়ামা",
        "কাজাখ",
        "কালাল্লিসুত",
        "সেন্ট্রাল খেমার",
        "কন্নড়",
        "কোরিয়ান",
        "কানুরি",
        "কাশ্মীরি",
        "কুর্দি",
        "কোমি",
        "কর্নিশ",
        "কিরঘিজ",
        "ল্যাটিন",
        "লাক্সেমবার্গিশ",
        "গান্ডা",
        "লিম্বুরগান",
        "লিঙ্গালা",
        "লাও",
        "লিথুয়ানিয়ান",
        "লুবা-কাটাঙ্গা",
        "লাটভিয়ান",
        "মালাগাসি",
        "মার্শালিজ",
        "মাওরি",
        "ম্যাসিডোনিয়ান",
        "মালয়ালম",
        "মঙ্গোলিয়ান",
        "মারাঠি",
        "মালয়",
        "মালটিজ",
        "বর্মী",
        "নাউরু",
        "উত্তর নেদেবেলে",
        "নেপালি",
        "এনডোঙ্গা",
        "ডাচ",
        "নরওয়েজিয়ান নাইনরস্ক",
        "নরওয়েজীয়",
        "দক্ষিণ নেদেবেলে",
        "নাভাজো",
        "চিচেওয়া",
        "অক্সিটান",
        "ওজিবওয়া",
        "ওরোমো",
        "ওড়িয়া",
        "ওসেশিয়ান",
        "পাঞ্জাবি",
        "পালি",
        "পোলিশ",
        "ধাক্কা",
        "পর্তুগীজ",
        "কেচুয়া",
        "রোমানশ",
        "রুন্ডি",
        "রোমানিয়ান",
        "রাশিয়ান",
        "কিনিয়ারওয়ান্ডা",
        "সংস্কৃত",
        "সার্ডিনিয়ান",
        "সিন্ধি",
        "উত্তর সামি",
        "সাঙ্গো",
        "সিংহল",
        "স্লোভাক",
        "স্লোভেনীয়",
        "সামোয়ান",
        "শোনা",
        "সোমালি",
        "আলবেনিয়ান",
        "সার্বিয়ান",
        "স্বাতী",
        "সোথো, দক্ষিণ",
        "সুদানিজ",
        "সুইডিশ",
        "সোয়াহিলি",
        "তামিল",
        "তেলেগু",
        "তাজিক",
        "থাই",
        "টাইগ্রিনিয়া",
        "তুর্কমেন",
        "তাগালগ",
        "সোয়ানা",
        "টোঙ্গা",
        "তুর্কি",
        "সোঙ্গা",
        "তাতার",
        "টুই",
        "তাহিতিয়ান",
        "উইঘুর",
        "ইউক্রেনীয়",
        "উর্দু",
        "উজবেক",
        "ভেন্দা",
        "ভিয়েতনামী",
        "ওয়ালুন",
        "ওলোফ",
        "জোসা",
        "ইদ্দিশ",
        "ইয়োরুবা",
        "ঝুয়াং",
        "চীনা",
        "জুলু",
    )

    first_names_male_common = (
        "অর্ক",
        "আকাশ",
        "আরিয়ান",
        "আদি",
        "অভিষেক",
        "অভি",
        "আনন্দ",
        "আবির",
        "ইমন",
        "চয়ন",
        "চঞ্চল",
        "তন্ময়",
        "তনয়",
        "তুষার",
        "নয়ন",
        "প্রান্ত",
        "প্রিতম",
        "প্রিয়ম",
        "প্রিয়",
        "প্রত্যয়",
        "বাদল",
        "মিলন",
        "রাহুল",
        "রোহিত",
        "লিটন",
        "শাওন",
        "শান্ত",
        "শুভ",
        "সজীব",
        "রাজ",
        "রাজু",
        "রুদ্র",
    )

    first_names_male_hinduism = (
        "অর্ঘ্য",
        "অশোক",
        "অজিত",
        "অর্ণব",
        "অক্ষয়",
        "অমল",
        "অজয়",
        "আশীষ",
        "আশুতোষ",
        "আয়ুষ",
        "কুনাল",
        "জয়ন্ত",
        "জয়দীপ",
        "জগদীশ",
        "প্রদ্যুম্ন",
        "প্রদীপ",
        "প্রশান্ত",
        "বিনয়",
        "বিরাট",
        "মৃনাল",
        "মৃত্যুঞ্জয়",
        "মনোজ",
        "শেখর",
        "সুশান্ত",
        "সৌমিক",
        "সৌম্য",
    )

    first_names_male_islamic = (
        "আবু",
        "আতাহার",
        "আজাদ",
        "আসাদ",
        "আনিস",
        "আজম",
        "আব্বাস",
        "ইকবাল",
        "ইউসুফ",
        "ইশতিয়াক",
        "ইমতিয়াজ",
        "ইজাজ",
        "এনামুল",
        "একরামুল",
        "কাফি",
        "করিম",
        "তামিম",
        "নাদিম",
        "নাইম",
        "বাকের",
        "বাসির",
        "মুনতাসির",
        "মুনতাকিম",
        "মোস্তাফিজ",
        "মুশফিক",
        "রায়হান",
        "রহিম",
        "রাশেদ",
        "রাসেল",
        "রাশেদুল",
        "শাহাবাজ",
        "শাহজাহান",
        "শহিদুল",
        "সাবের",
        "সাব্বির",
    )

    first_names_female_common = (
        "অর্পিতা",
        "অঞ্জনা",
        "অহনা",
        "অন্তরা",
        "অর্না",
        "অনন্যা",
        "আরিয়া",
        "আশা",
        "আলিয়া",
        "ইশিতা",
        "কেয়া",
        "কবিতা",
        "কাজল",
        "খুশি",
        "ডলি",
        "জনা",
        "নন্দিতা",
        "নিশিতা",
        "প্রীতি",
        "প্রিয়তি",
        "প্রিয়াঙ্কা",
        "প্রিয়া",
        "বাঁধন",
        "বৃষ্টি",
        "বিনা",
        "বিপাশা",
        "মিথিলা",
        "মিষ্টি",
        "মিলা",
        "মিনা",
        "মিম",
        "রিনা",
        "লতা",
        "শ্রাবনী",
        "শ্রাবন্তী",
        "সুরভি",
    )

    first_names_female_hinduism = (
        "অদৃতা",
        "অনিন্দিতা",
        "অলোকা",
        "অদিতি",
        "আমায়া",
        "আরাধ্যা",
        "আরুণি",
        "আশালতা",
        "আশ্বিনী",
        "আয়ুশি",
        "ঋষিতা",
        "ঈশানি",
        "কাবেরি",
        "দূর্গা",
        "বিদ্যা",
        "মাধুরী",
        "মাধু",
        "হৈমন্তী",
        "শুভশ্রী",
    )

    first_names_female_islamic = (
        "আক্তারা",
        "আফিয়া",
        "আসিফা",
        "আফিফা",
        "আফসানা",
        "আয়েশা",
        "জোবায়দা",
        "তাসফিয়া",
        "তাসনিম",
        "তামান্না",
        "নুসরাত",
        "ফৌজিয়া",
        "ফারিহা",
        "মেহজাবিন",
        "মোনালিসা",
        "মালিহা",
        "রাজিয়া",
        "রোজিনা",
        "শারমিন",
        "সানজিদা",
        "সুমাইয়া",
    )

    last_names_common = (
        "চৌধুরী",
        "তালুকদার",
        "প্রামানিক",
        "বিশ্বাস",
        "মৃধা",
        "মজুমদার",
        "মোড়ল",
        "মন্ডল",
        "সরকার",
        "সিনহা",
    )

    last_names_hinduism = (
        "আচার্য্য",
        "কুমার",
        "কান্ত",
        "গাঙ্গুলি",
        "গঙ্গোপাধ্যায়",
        "ঘোষ",
        "চ্যাটার্জি",
        "চট্টোপাধ্যায়",
        "চন্দ্র",
        "ঠাকুর",
        "দত্ত",
        "দাস",
        "দেব",
        "দে",
        "দাশগুপ্তা",
        "পাল",
        "পোদ্দার",
        "পাণ্ডে",
        "প্রধান",
        "ব্যানার্জি",
        "বন্দোপাধ্যায়",
        "বোস",
        "বসু",
        "বর্মন",
        "বাগচী",
        "মুখার্জি",
        "মিশ্র",
        "মিত্র",
        "যাদব",
        "শুক্লা",
        "সাহা",
        "সিং",
        "সেন",
        "রায়",
        "রাও",
    )

    last_names_islamic = (
        "আলি",
        "আক্তার",
        "আওয়াল",
        "আলম",
        "আবদুল্লাহ",
        "ইসলাম",
        "উদ্দিন",
        "কাদের",
        "খান",
        "জামান",
        "মিঞা",
        "হোসাইন",
        "হক",
        "হুরাইরা",
        "হাকিম",
        "রহমান",
    )

    last_names_female_islamic = (
        "আরা",
        "খানম",
        "খাতুন",
        "জাহান",
        "তাবাসসুম",
        "বেগম",
        "সুলতানা",
    ) + last_names_islamic

    formats_male = [
        "{{first_name_male_common}} {{last_name_common}}",
        "{{first_name_male_hinduism}} {{last_name_common}}",
        "{{first_name_male_common}} {{last_name_hinduism}}",
        "{{first_name_male_hinduism}} {{last_name_hinduism}}",
        "{{first_name_male_islamic}} {{last_name_common}}",
        "{{first_name_male_common}} {{last_name_islamic}}",
        "{{first_name_male_islamic}} {{last_name_islamic}}",
    ]

    formats_female = [
        "{{first_name_female_common}} {{last_name_common}}",
        "{{first_name_female_hinduism}} {{last_name_common}}",
        "{{first_name_female_common}} {{last_name_hinduism}}",
        "{{first_name_female_hinduism}} {{last_name_hinduism}}",
        "{{first_name_female_islamic}} {{last_name_common}}",
        "{{first_name_female_common}} {{last_name_female_islamic}}",
        "{{first_name_female_islamic}} {{last_name_female_islamic}}",
    ]

    formats = formats_male + formats_female

    first_names_male = first_names_male_common + first_names_male_hinduism + first_names_male_islamic
    first_names_female = first_names_female_common + first_names_female_hinduism + first_names_female_islamic
    first_names = first_names_male + first_names_female

    last_names_male = last_names_common + last_names_hinduism + last_names_islamic
    last_names_female = last_names_common + last_names_hinduism + last_names_female_islamic
    last_names = last_names_male + last_names_female

    def first_name_male_common(self) -> str:
        """
        Return religiously unbiased male first name.
        :example: 'প্রিতম'
        """
        return self.random_element(self.first_names_male_common)

    def first_name_male_hinduism(self) -> str:
        """
        Return Hindu religion based male first name.
        :example: 'অশোক'
        """
        return self.random_element(self.first_names_male_hinduism)

    def first_name_male_islamic(self) -> str:
        """
        Return Islam religion based male first name.
        :example: 'ইকবাল'
        """
        return self.random_element(self.first_names_male_islamic)

    def first_name_female_common(self) -> str:
        """
        Return religiously unbiased female first name.
        :example: 'অর্পিতা'
        """
        return self.random_element(self.first_names_female_common)

    def first_name_female_hinduism(self) -> str:
        """
        Return Hindu religion based female first name.
        :example: 'দূর্গা'
        """
        return self.random_element(self.first_names_female_hinduism)

    def first_name_female_islamic(self) -> str:
        """
        Return Islam religion based female first name.
        :example: 'মেহজাবিন'
        """
        return self.random_element(self.first_names_male_islamic)

    def last_name_common(self) -> str:
        """
        Return religiously and gender unbiased last name.
        :example: 'চৌধুরী'
        """
        return self.random_element(self.last_names_common)

    def last_name_hinduism(self) -> str:
        """
        Return gender unbiased Hindu religion based last name.
        :example: 'দত্ত'
        """
        return self.random_element(self.last_names_hinduism)

    def last_name_islamic(self) -> str:
        """
        Return gender unbiased Islam religion based last name.
        :example: 'আলি'
        """
        return self.random_element(self.last_names_islamic)

    def last_name_female_islamic(self) -> str:
        """
        Return Islam religion based female last name.
        :example: 'খাতুন'
        """
        return self.random_element(self.last_names_female_islamic)



from .. import Provider as PersonProvider


class Provider(PersonProvider):
    formats_female = (
        "{{first_name_female}} {{last_name}}",
        "{{first_name_unisex}} {{last_name}}",
        "{{prefix_female}} {{first_name_unisex}} {{last_name}}",
        "{{prefix_female}} {{first_name_female}} {{last_name}}",
    )
    formats_male = (
        "{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{middle_name}} {{last_name}}",
        "{{first_name_unisex}} {{middle_name}} {{last_name}}",
        "{{prefix_male}} {{first_name_male}} {{last_name}}",
    )
    formats = formats_female + formats_male

    # Name from : https://en.wikipedia.org/wiki/Vietnamese_name
    # and https://vinpearl.com/en/vietnamese-names-top-200-popular-names-for-boys-and-girls

    first_names_female = (
        "Ngọc",
        "Hương",
        "Lan",
        "Mai",
        "Thảo",
        "Linh",
        "Hồng",
        "Chi",
        "Vân",
        "Duyên",
        "Dương",
        "Yến",
        "Vi",
        "Ánh",
        "Xuân",
    )

    first_names_unisex = (
        "An",
        "Hà",
        "Bảo",
        "Lâm",
        "Hạnh",
        "Thành",
        "Kim",
        "Nhật",
        "Phương",
        "Khoa",
        "Hải",
        "Nhật",
    )

    first_names_male = (
        "Nam",
        "Hưng",
        "Vũ",
        "Tú",
        "Hoàng",
        "Phúc",
        "Trung",
        "Quang",
        "Anh",
        "Khoa",
        "Dũng",
        "Quang",
        "Thành",
        "Huy",
        "Bảo",
        "Châu",
        "Minh",
        "Tùng",
        "Nhiên",
        "Trọng",
    )

    middle_names = (
        "Văn",
        "Thị",
        "Quang",
        "Đức",
        "Trí",
        "Xuân",
        "Hoàng",
        "Hải",
        "Đức",
        "Thế",
        "Tấn",
        "Phú",
        "Hữu",
        "Bảo",
        "Mai",
        "Mai Bảo",
    )

    last_names = ("Nguyễn", "Trần", "Lê", "Phạm", "Vũ", "Đặng", "Bùi", "Dương", "Mai", "Hoàng")

    # Typically, Vietnamese will be addressed with their given name and a prefix
    # https://en.wikipedia.org/wiki/Vietnamese_name#Given_name

    prefixes_female = ("Cô", "Chị", "Bà", "Quý cô")

    prefixes_male = ("Ông", "Anh", "Bác", "Quý ông")

    def first_name_unisex(self) -> str:
        return self.random_element(self.first_names_unisex)

    def middle_name(self) -> str:
        return self.random_element(self.middle_names)



import math
import string
import sys
import warnings

from decimal import Decimal
from enum import Enum
from typing import Any, Dict, Iterable, Iterator, List, Optional, Set, Tuple, Type, TypeVar, Union, cast, no_type_check

from faker.typing import BasicNumber

from ...exceptions import BaseFakerException
from .. import BaseProvider, ElementsType

TypesNames = List[str]
TypesSpec = Union[List[Type], Tuple[Type, ...]]
TEnum = TypeVar("TEnum", bound=Enum)


class EmptyEnumException(BaseFakerException):
    pass


class Provider(BaseProvider):
    default_value_types: ElementsType[str] = (
        "str",
        "str",
        "str",
        "str",
        "float",
        "int",
        "int",
        "decimal",
        "date_time",
        "uri",
        "email",
    )

    def _check_signature(self, value_types: Optional[TypesSpec], allowed_types: Optional[TypesSpec]) -> TypesSpec:
        if value_types is not None and not isinstance(value_types, (list, tuple)):
            value_types = (value_types,)
            warnings.warn(
                "Passing `value_types` as positional arguments is going to be "
                "deprecated.  Pass them as a list or tuple instead.",
                PendingDeprecationWarning,
            )
        if allowed_types is not None and not isinstance(allowed_types, (list, tuple)):
            allowed_types = (allowed_types,)
            warnings.warn(
                "Passing `allowed_types` as positional arguments is going to be "
                "deprecated.  Pass them as a list or tuple instead.",
                PendingDeprecationWarning,
            )
        if value_types is None:
            value_types = ()
        if allowed_types is None:
            allowed_types = ()
        return tuple(value_types) + tuple(allowed_types)

    def pyobject(
        self,
        object_type: Optional[Type[Union[bool, str, float, int, tuple, set, list, Iterable, dict]]] = None,
    ) -> Optional[Union[bool, str, float, int, tuple, set, list, Iterable, dict]]:
        """
        Generates a random object passing the type desired.

        :object_type: the type of the object to generate.
        :return: the random object generated.
        :raises ValueError: if the object type passed is not supported
        """
        if object_type is None:
            return None
        elif object_type == bool:
            return self.pybool()
        elif object_type == str:
            return self.pystr()
        elif object_type == float:
            return self.pyfloat()
        elif object_type == int:
            return self.pyint()
        elif object_type == tuple:
            return self.pytuple()
        elif object_type == set:
            return self.pyset()
        elif object_type == list:
            return self.pylist()
        elif object_type == Iterable:
            return self.pyiterable()
        elif object_type == dict:
            return self.pydict()
        else:
            raise ValueError(f"Object type `{object_type}` is not supported by `pyobject` function")

    def pybool(self, truth_probability: int = 50) -> bool:
        """
        Generates a random boolean, optionally biased towards `True` or `False`.

        :truth_probability: Probability of generating a `True` value. Must be between `0` and `100` inclusive'.
        :return: Random boolean.
        :raises ValueError: If invalid `truth_probability` is provided.
        """
        if truth_probability < 0 or truth_probability > 100:
            raise ValueError("Invalid `truth_probability` value: must be between `0` and `100` inclusive")

        return self.random_int(1, 100) <= truth_probability

    def pystr(
        self,
        min_chars: Optional[int] = None,
        max_chars: int = 20,
        prefix: str = "",
        suffix: str = "",
    ) -> str:
        """
        Generates a random string of upper and lowercase letters.

        :param min_chars: minimum length of the random part.
        :param max_chars: maximum length of the random part.
        :param prefix: an optional prefix to prepend to the random string.
        :param suffix: an optional suffix to append to the random string.
        :return: Random of random length between min and max characters.
        """
        if min_chars is None:
            chars = "".join(self.random_letters(length=max_chars))
        else:
            assert max_chars >= min_chars, "Maximum length must be greater than or equal to minimum length"
            chars = "".join(
                self.random_letters(
                    length=self.generator.random.randint(min_chars, max_chars),
                ),
            )

        return prefix + chars + suffix

    def pystr_format(
        self,
        string_format: str = "?#-###{{random_int}}{{random_letter}}",
        letters: str = string.ascii_letters,
    ) -> str:
        return self.bothify(self.generator.parse(string_format), letters=letters)

    @no_type_check
    def pyfloat(
        self,
        left_digits: Optional[int] = None,
        right_digits: Optional[int] = None,
        positive: Optional[bool] = None,
        min_value: Optional[Union[float, int]] = None,
        max_value: Optional[Union[float, int]] = None,
    ) -> float:
        if left_digits is not None and left_digits < 0:
            raise ValueError("A float number cannot have less than 0 digits in its integer part")
        if right_digits is not None and right_digits < 0:
            raise ValueError("A float number cannot have less than 0 digits in its fractional part")
        if left_digits == 0 and right_digits == 0:
            raise ValueError("A float number cannot have less than 0 digits in total")
        if min_value is not None and max_value is not None:
            if min_value > max_value:
                raise ValueError("Min value cannot be greater than max value")
        if None not in (min_value, max_value) and min_value == max_value:
            raise ValueError("Min and max value cannot be the same")
        if positive and min_value is not None and min_value <= 0:
            raise ValueError("Cannot combine positive=True with negative or zero min_value")
        if left_digits is not None and max_value and math.ceil(math.log10(abs(max_value))) > left_digits:
            raise ValueError("Max value must fit within left digits")
        if left_digits is not None and min_value and math.ceil(math.log10(abs(min_value))) > left_digits:
            raise ValueError("Min value must fit within left digits")

        # Make sure at least either left or right is set
        if left_digits is None and right_digits is None:
            needed_left_digits = max(1, math.ceil(math.log10(max(abs(max_value or 1), abs(min_value or 1)))))
            right_digits = self.random_int(1, sys.float_info.dig - needed_left_digits)

        # If only one side is set, choose #digits for other side
        if (left_digits is None) ^ (right_digits is None):
            if left_digits is None:
                left_digits = max(1, sys.float_info.dig - right_digits)
            else:
                right_digits = max(1, sys.float_info.dig - left_digits)

        # Make sure we don't ask for too many digits!
        if left_digits + right_digits > sys.float_info.dig:
            raise ValueError(
                f"Asking for too many digits ({left_digits} + {right_digits} == {left_digits + right_digits} > "
                f"{sys.float_info.dig})",
            )

        sign = ""
        if (min_value is not None) or (max_value is not None):
            # Copy values to ensure we're not modifying the original values and thus going out of bounds
            left_min_value = min_value
            left_max_value = max_value
            # Make sure left_digits still respected
            if left_digits is not None:
                if max_value is None:
                    left_max_value = 10**left_digits  # minus smallest representable, adjusted later
                if min_value is None:
                    left_min_value = -(10**left_digits)  # plus smallest representable, adjusted later

            if max_value is not None and max_value < 0:
                left_max_value += 1  # as the random_int will be generated up to max_value - 1
            if min_value is not None and min_value < 0:
                left_min_value += 1  # as we then append digits after the left_number
            left_number = self._safe_random_int(
                left_min_value,
                left_max_value,
                positive,
            )
        else:
            if positive is None:
                sign = self.random_element(("+", "-"))
            elif positive is True:
                sign = "+"
            else:
                sign = "-"

            left_number = self.random_number(left_digits)

        result = float(f"{sign}{left_number}.{self.random_number(right_digits)}")
        if positive and result == 0:
            if right_digits:
                result = float("0." + "0" * (right_digits - 1) + "1")
            else:
                result += sys.float_info.epsilon

        if right_digits:
            result = min(result, 10**left_digits - float(f'0.{"0" * (right_digits - 1)}1'))
            result = max(result, -(10**left_digits + float(f'0.{"0" * (right_digits - 1)}1')))
        else:
            result = min(result, 10**left_digits - 1)
            result = max(result, -(10**left_digits + 1))

        # It's possible for the result to end up > than max_value or < than min_value
        # When this happens we introduce some variance so we're not always exactly at the boundary.
        # When only one bound is set, use the exceeded amount as the variance bound.
        # When positive=True without explicit min_value, treat min_value as 0.
        effective_min = 0 if (positive and min_value is None) else min_value
        if max_value is not None and result > max_value:
            exceeded_by = result - max_value
            if effective_min is not None:
                variance_bound = max_value - effective_min
            else:
                variance_bound = exceeded_by
            result = result - (exceeded_by + self.generator.random.uniform(0, variance_bound))
        if effective_min is not None and result < effective_min:
            below_by = effective_min - result
            if max_value is not None:
                variance_bound = max_value - effective_min
            else:
                variance_bound = below_by
            result = result + (below_by + self.generator.random.uniform(0, variance_bound))

        return result

    def _safe_random_int(self, min_value: float, max_value: float, positive: bool) -> int:
        orig_min_value = min_value
        orig_max_value = max_value

        if min_value is None:
            min_value = max_value - self.random_int()
        if max_value is None:
            max_value = min_value + self.random_int()
        if positive:
            min_value = max(min_value, 0)

        if min_value == max_value:
            return self._safe_random_int(orig_min_value, orig_max_value, positive)
        else:
            min_value = int(min_value)
            max_value = int(max_value - 1)
            if max_value < min_value:
                max_value += 1
            return self.random_int(min_value, max_value)

    def pyint(self, min_value: int = 0, max_value: int = 9999, step: int = 1) -> int:
        return self.generator.random_int(min_value, max_value, step=step)

    def _random_int_of_length(self, length: int) -> int:
        """Generate a random integer of a given length

        If length is 0, so is the number. Otherwise the first digit must not be 0.
        """

        if length < 0:
            raise ValueError("Length must be a non-negative integer.")
        elif length == 0:
            return 0
        else:
            min_value = 10 ** (length - 1)
            max_value = (10**length) - 1
            return self.pyint(min_value=min_value, max_value=max_value)

    def pydecimal(
        self,
        left_digits: Optional[int] = None,
        right_digits: Optional[int] = None,
        positive: Optional[bool] = None,
        min_value: Optional[BasicNumber] = None,
        max_value: Optional[BasicNumber] = None,
    ) -> Decimal:
        if left_digits is not None and left_digits < 0:
            raise ValueError("A decimal number cannot have less than 0 digits in its integer part")
        if right_digits is not None and right_digits < 0:
            raise ValueError("A decimal number cannot have less than 0 digits in its fractional part")
        if (left_digits is not None and left_digits == 0) and (right_digits is not None and right_digits == 0):
            raise ValueError("A decimal number cannot have 0 digits in total")
        if min_value is not None and max_value is not None and min_value > max_value:
            raise ValueError("Min value cannot be greater than max value")
        if min_value is not None and max_value is not None and min_value == max_value:
            raise ValueError("Min and max value cannot be the same")
        if positive and min_value is not None and min_value <= 0:
            raise ValueError("Cannot combine positive=True with negative or zero min_value")
        if (
            left_digits is not None
            and max_value
            and math.ceil(math.log10(abs(max_value))) > left_digits  # type: ignore[arg-type]
        ):
            raise ValueError("Max value must fit within left digits")
        if (
            left_digits is not None
            and min_value
            and math.ceil(math.log10(abs(min_value))) > left_digits  # type: ignore[arg-type]
        ):
            raise ValueError("Min value must fit within left digits")

        # if either left or right digits are not specified we randomly choose a length
        max_random_digits = 100
        # Because if min_value is bigger than 10**100
        max_digits_from_value = max(
            math.ceil(math.log10(abs(min_value or 1))),  # type: ignore[arg-type]
            math.ceil(math.log10(abs(max_value or 1))),  # type: ignore[arg-type]
        )
        max_left_random_digits = max(max_random_digits, max_digits_from_value + 10)

        if min_value is not None and min_value >= 0:
            sign = "+"
        elif max_value is not None and max_value <= 0:
            sign = "-"
        else:
            if positive is None:
                sign = self.random_element(("+", "-"))
            else:
                sign = "+" if positive else "-"

        if sign == "+":
            if max_value is not None:
                left_number = str(self.random_int(int(max(min_value or 0, 0)), int(max_value)))
            else:
                min_left_digits = math.ceil(math.log10(max(min_value or 1, 1)))
                if left_digits is None:
                    left_digits = self.random_int(min_left_digits, max_left_random_digits)
                left_number = str(self._random_int_of_length(left_digits))
        else:
            if min_value is not None:
                left_number = str(
                    self.random_int(
                        int(abs(min(max_value or 0, 0))),  # type: ignore[arg-type,call-overload]
                        int(abs(min_value)),  # type: ignore[arg-type,call-overload]
                    )
                )
            else:
                min_left_digits = math.ceil(math.log10(abs(min(max_value or 1, 1))))  # type: ignore[arg-type]
                if left_digits is None:
                    left_digits = self.random_int(min_left_digits, max_left_random_digits)
                left_number = str(self._random_int_of_length(left_digits))

        if right_digits is None:
            right_digits = self.random_int(0, max_random_digits)

        right_number = "".join([str(self.random_digit()) for _ in range(0, right_digits)])

        result = Decimal(f"{sign}{left_number}.{right_number}")

        # Because the random result might have the same number of decimals as max_value the random number
        # might be above max_value or below min_value
        if max_value is not None and result > max_value:
            result = Decimal(str(max_value))
        if min_value is not None and result < min_value:
            result = Decimal(str(min_value))

        return result

    def pytuple(
        self,
        nb_elements: int = 10,
        variable_nb_elements: bool = True,
        value_types: Optional[TypesSpec] = None,
        allowed_types: Optional[TypesSpec] = None,
    ) -> Tuple[Any, ...]:
        return tuple(
            self._pyiterable(
                nb_elements=nb_elements,
                variable_nb_elements=variable_nb_elements,
                value_types=value_types,
                allowed_types=allowed_types,
            )
        )

    def pyset(
        self,
        nb_elements: int = 10,
        variable_nb_elements: bool = True,
        value_types: Optional[TypesSpec] = None,
        allowed_types: Optional[TypesSpec] = None,
    ) -> Set[Any]:
        return set(
            self._pyiterable(
                nb_elements=nb_elements,
                variable_nb_elements=variable_nb_elements,
                value_types=value_types,
                allowed_types=allowed_types,
            )
        )

    def pylist(
        self,
        nb_elements: int = 10,
        variable_nb_elements: bool = True,
        value_types: Optional[TypesSpec] = None,
        allowed_types: Optional[TypesSpec] = None,
    ) -> List[Any]:
        return list(
            self._pyiterable(
                nb_elements=nb_elements,
                variable_nb_elements=variable_nb_elements,
                value_types=value_types,
                allowed_types=allowed_types,
            )
        )

    @no_type_check
    def pyiterable(
        self,
        nb_elements: int = 10,
        variable_nb_elements: bool = True,
        value_types: Optional[TypesSpec] = None,
        allowed_types: Optional[TypesSpec] = None,
    ) -> Iterable[Any]:
        value_types: TypesSpec = self._check_signature(value_types, allowed_types)
        return self.random_element([self.pylist, self.pytuple, self.pyset])(
            nb_elements=nb_elements,
            variable_nb_elements=variable_nb_elements,
            value_types=value_types,
            allowed_types=allowed_types,
        )

    def _random_type(self, type_list: List[str]) -> str:
        value_type: str = self.random_element(type_list)

        method_name = f"py{value_type}"
        if hasattr(self, method_name):
            value_type = method_name

        return self.generator.format(value_type)

    def _pyiterable(
        self,
        nb_elements: int = 10,
        variable_nb_elements: bool = True,
        value_types: Optional[TypesSpec] = None,
        allowed_types: Optional[TypesSpec] = None,
    ) -> Iterator:
        value_types: TypesSpec = self._check_signature(value_types, allowed_types)

        value_types: TypesNames = [
            t if isinstance(t, str) else getattr(t, "__name__", type(t).__name__).lower()
            for t in value_types
            # avoid recursion
            if t not in ["iterable", "list", "tuple", "dict", "set"]
        ]
        if not value_types:
            value_types = self.default_value_types  # type: ignore

        if variable_nb_elements:
            nb_elements = self.randomize_nb_elements(nb_elements, min=1)

        for _ in range(nb_elements):
            yield self._random_type(value_types)

    def pydict(
        self,
        nb_elements: int = 10,
        variable_nb_elements: bool = True,
        value_types: Optional[TypesSpec] = None,
        allowed_types: Optional[TypesSpec] = None,
    ) -> Dict[Any, Any]:
        """
        Returns a dictionary.

        :nb_elements: number of elements for dictionary
        :variable_nb_elements: is use variable number of elements for dictionary
        :value_types: type of dictionary values
        """

        words_list_count = len(self.generator.get_words_list())

        if variable_nb_elements:
            nb_elements = self.randomize_nb_elements(nb_elements, min=1)

        if nb_elements > words_list_count:
            warnings.warn(
                f"Number of nb_elements is greater than the number of words in the list."
                f" {words_list_count} words will be used.",
                RuntimeWarning,
            )
            nb_elements = words_list_count

        return dict(
            zip(
                self.generator.words(nb_elements, unique=True),
                self._pyiterable(
                    nb_elements=nb_elements,
                    variable_nb_elements=False,
                    value_types=value_types,
                    allowed_types=allowed_types,
                ),
            )
        )

    def pystruct(
        self,
        count: int = 10,
        value_types: Optional[TypesSpec] = None,
        allowed_types: Optional[TypesSpec] = None,
    ) -> Tuple[List, Dict, Dict]:
        value_types: TypesSpec = self._check_signature(value_types, allowed_types)

        value_types: TypesNames = [
            t if isinstance(t, str) else getattr(t, "__name__", type(t).__name__).lower()
            for t in value_types
            # avoid recursion
            if t != "struct"
        ]
        if not value_types:
            value_types = self.default_value_types  # type: ignore

        types = []
        d = {}
        nd = {}
        for i in range(count):
            d[self.generator.word()] = self._random_type(value_types)
            types.append(self._random_type(value_types))
            nd[self.generator.word()] = {
                i: self._random_type(value_types),
                i
                + 1: [
                    self._random_type(value_types),
                    self._random_type(value_types),
                    self._random_type(value_types),
                ],
                i
                + 2: {
                    i: self._random_type(value_types),
                    i + 1: self._random_type(value_types),
                    i
                    + 2: [
                        self._random_type(value_types),
                        self._random_type(value_types),
                    ],
                },
            }
        return types, d, nd

    def enum(self, enum_cls: Type[TEnum]) -> TEnum:
        """
        Returns a random enum of the provided input `Enum` type.

        :param enum_cls: The `Enum` type to produce the value for.
        :returns: A randomly selected enum value.
        """

        if enum_cls is None:
            raise ValueError("'enum_cls' cannot be None")

        if not issubclass(enum_cls, Enum):
            raise TypeError("'enum_cls' must be an Enum type")

        members: List[TEnum] = list(cast(Iterable[TEnum], enum_cls))

        if len(members) < 1:
            raise EmptyEnumException(f"The provided Enum: '{enum_cls.__name__}' has no members.")

        return self.random_element(members)



from .. import Provider as SsnProvider


def checksum(s: str) -> int:
    def _get_alphabet_weight(c: str) -> int:
        """A=10, B=11, ...., H=17,
        I=34,
        J=18, K=19, ..., N=22,
        O=35,
        P=23, Q=24, ..., V=29,
        W=32,
        X=30, Y=31, Z=33
        """
        if ord(c) < 73:  # A-H
            return ord(c) - 55
        if ord(c) == 73:  # I
            return ord(c) - 39
        if ord(c) < 79:  # J-N
            return ord(c) - 56
        if ord(c) == 79:  # O
            return ord(c) - 44
        if ord(c) < 87:  # P-V
            return ord(c) - 57
        if ord(c) == 87:  # W
            return ord(c) - 55
        if ord(c) < 90:  # X, Y
            return ord(c) - 58
        return ord(c) - 57  # Z

    res = 0
    for i, c in enumerate(s):
        if i == 0:
            res = _get_alphabet_weight(c) % 10 * 9 + _get_alphabet_weight(c) // 10
        elif i < 9:
            res += int(c) * (9 - i)
        else:
            res += int(c)
    return res


class Provider(SsnProvider):
    def ssn(self) -> str:
        ssn_without_last_char = self.numerify(self.random_uppercase_letter() + str(self.random_int(1, 2)) + "#######")
        last_char = str((10 - checksum(ssn_without_last_char) % 10) % 10)
        return ssn_without_last_char + last_char



from typing import Dict

from .. import Provider as LoremProvider


class Provider(LoremProvider):
    """Implement lorem provider for ``he_IL`` locale."""

    word_list = (
        "אאוגו",
        "אגת",
        "אדיפיסינג",
        "אדנדום",
        "אט",
        "איאקוליס",
        "איבן",
        "איף",
        "איפסום",
        "אלית",
        "אלמנקום",
        "אמט",
        "אס",
        "אפאח",
        "אקווזמן",
        "ארווס",
        "בגורמי",
        "בורק?",
        "בלובק",
        "בלינדו",
        "בלינך",
        "בליקרה",
        "בעליק",
        "בעריר",
        "בראיט",
        "ברומץ",
        "בריקנה",
        "ברשג",
        "גדדיש",
        "גולר",
        "גק",
        "דול",
        "דולור",
        "דומור",
        "דז",
        "דיאם",
        "דלאמת",
        "דס",
        "הבקיץ",
        "הדש",
        "הועניב",
        "היושבב",
        "הכייר",
        "השמה",
        "התידם",
        "וואל",
        "וולופטה",
        "וחאית",
        "ולחת",
        "ולתיעם",
        "ומעיוט",
        "ומרגשח",
        "וסטיבולום",
        "וסתעד",
        "וק",
        "ותלברו",
        "זותה",
        "חשלו",
        "טידום",
        "יבש",
        "יהול",
        "ישבעס",
        "כאנה",
        "כלרשט",
        "להאמית",
        "לורם",
        "ליאמום",
        "ליבם",
        "ליץ",
        "לכימפו",
        "לכנו",
        "לכנוץ",
        "למטכין",
        "למרקוח",
        "למרקל",
        "לפמעט",
        "לפריקך",
        "לפתיעם",
        "לקטוס",
        "לרטי",
        "לתיג",
        "לתכי",
        "מא",
        "מגמש",
        "מונחף",
        "מונפרד",
        "מונפרר",
        "מוסן",
        "מורגם",
        "מיחוצים",
        "מנורך",
        "מנכם",
        "מנק",
        "מנת",
        "מרגשי",
        "נובש",
        "נולום",
        "נון",
        "נונסטי",
        "ניבאה",
        "ניסי",
        "ניצאחו",
        "נמרגי",
        "נשואי",
        "סאפיאן",
        "סוברט",
        "סולגק",
        "סוליסי",
        "סחטיר",
        "סטום",
        "סיט",
        "סילקוף",
        "סכעיט",
        "סת",
        "סתשם",
        "עמחליף",
        "ערששף",
        "פוסיליס",
        "צוט",
        "צופעט",
        "צורק",
        "קוויז",
        "קוויס",
        "קולהע",
        "קולורס",
        "קונדימנטום",
        "קונסקטורר",
        "קורוס",
        "קלאצי",
        "קלובר",
        "קראס",
        "קרהשק",
        "רוגצה",
        "שבצק",
        "שהכים",
        "שלושע",
        "שמחויט",
        "שנרא",
        "שעותלשך",
        "שערש",
        "תוק",
        "תצטנפל",
        "תצטריק",
        "תרבנך",
    )

    parts_of_speech: Dict[str, tuple] = {}



from .. import Provider as InternetProvider


class Provider(InternetProvider):
    safe_email_tlds = ("com", "net", "pt", "pt")
    free_email_domains = ("gmail.com", "hotmail.com", "clix.pt", "sapo.pt")
    tlds = ("com", "com", "com", "net", "org", "pt", "pt", "pt")
